/*
	The Actions datum is meant to be defined on all mobs which use the ActionHandler.  Actions exist as a series of things to
	do which can be found on mob.Actions[].

	Where I used to have hotkeys (access icons), abilities, and client screen icons, I am now attempting to merge all
	of this into the actions datum.

	Note that this is the best place to put a proc that will generate information related to client.screen visual updates
	because the information could be stored here for easy removal from client.screen.
*/

var/const/ACTION_STATUS_IMPOSSIBLE = 0
var/const/ACTION_STATUS_PENDING = 1
var/const/ACTION_STATUS_COMPLETE = 2

var/const/UNDEFINED_ACTION_ENERGY_COST = 1001
var/const/UNDEFINED_ACTION_MASS_COST = 1001

Action
	var/mob/actor // Who is doing it.
	// var/action // What's being done. // This is now replaced with the actual ActionType.
	var/atom/target // What are you doing it to.
	var/modifier // What are you doing it with (optional, e.g. swing a weapon)

	var/image/myImages[] // This is a list of images related to this action.  They are stored here for easy removal.

	var/massCost	// Cost in mass to use this action.
	var/energyCost	// Cost in energy to use this action.

	/* New()
		Takes 3 args in order of actor, target, modifier */
	New(var/newActor,var/newTarget,var/newModifier)

		DebugMsg("Null action created or flow reached root Action.")
		del src

	/* Attempt()
		Normally an action is attempted when it is encountered in the action queue, but if flow falls to the base action
		then we know that something went wrong.

		Each Action.Attempt() should:
		  1. Attempt to do modifierever the action, as appropriate, based off of modifier's programmed in Attempt()
		  2. Sleep() the appropriate amount of time representing the action.
		  3. Return one of three things: ACTION_STATUS_IMPOSSIBLE, ACTION_STATUS_PENDING, or ACTION_STATUS_COMPLETE. */

	proc/Attempt()

		if (actor != null)
			DebugMsg("[type].Attempt() undefined action activated by [actor].")
		else
			DebugMsg("[type].Attempt() undefined action activated.")
		return (ACTION_STATUS_IMPOSSIBLE)

	/* GetClientScreenImages()
		Returns a list of images representing this particular action.
		Returns null if there is none.
		Basically, should initialize and return myImages[] if it has some, otherwise return null. */
	proc/GetClientScreenImages()
		if (myImages != null)
			return (myImages)
		else
			return(null)

	proc/GetEnergyCost()
		if (energyCost == null)
			return (UNDEFINED_ACTION_ENERGY_COST)
		else
			return (energyCost)

	proc/GetMassCost()
		if (massCost == null)
			return (UNDEFINED_ACTION_MASS_COST)
		else
			return(massCost)

	proc/CannotAfford()
		if (GetMassCost() != UNDEFINED_ACTION_MASS_COST)
			if (GetMassCost() >= actor.GetMass())
				return(1)

		if (GetEnergyCost() != UNDEFINED_ACTION_ENERGY_COST)
			if (GetEnergyCost() >= actor.GetEnergy())
				return(1)

		return(0)
