/*
==================================================
	Project Xenoverse
	By Geldon Yetichsky
	Using the most excellent BYOND suite.
==================================================
actionQueue.dm
	Handling procedures and variable structures for the action queue model.
==================================================

Notes:
	This is to be coded in terms of modular programming methods.

Requires:
	ACTIONPOINTSREPLENISHEDPERTURN -- Global constant - Action Points Replenished Per Turn
	MAXIMUMACTIONPOINTS - Global constant - Maximum Action Points
	TIMEPERACTION - Global constant - Time per action in 1/10th a second

	mob needs to have an actionPoint variable defined.
	mob needs to have the actions variable defined as a list off of the Actions datum
	mob needs to have the interruptions variable defined as a list off of the Interruptions datum

	From here, you can customize it as appropriate to flavor of the game.
	For example, in this design, the mob needs the following procedures:
		GetMoveAPCost() -- Returns action point costs to move
		GetAttackAPCost() -- Returns action point cost to attack
	And there will be global procs set up to handle various things:
		MoveHandler -- Handles moving mobs
		AttackHandler -- Handles attacking mobs.
*/

var/const/TIMEPERACTION = 10
var/const/ACTIONPOINTSREPLENISHEDPERTURN = 1
var/const/MAXIMUMACTIONPOINTS = 10

/* Interruptions are things that have happened since the last turn that need to be resolved before the action queue is
   free to continue */

Interruption
	var/mob/actor // Who is being interrupted
	var/action // What are you interrupting with
	var/what // What are you doing it with.  (optiona, e.g. block with a shield.)
	var/atom/subject // What are you doing it to.

/* InterruptionHandler()
	Old, old code: I'll want to revamp this later.  For now, keep this in mind:
	return(0) unless want to continue from the top of the HeartBeat() when done */
proc/InterruptionHandler(var/mob/thisMob)

	if (thisMob.interruptions != null && thisMob.interruptions.len > 0)
		var/tmp/Interruption/thisInterruption = thisMob.interruptions[1]

		switch (thisInterruption.action)
			if ("Purge")
				thisMob.ActionHandlerProcessPurgeAll()

			if ("RemoveLast")
				thisMob.ActionHandlerProcessRemoveLast()

			if ("stun") // Stuns the mob for rounds equal to the .what arg.
				if (thisInterruption.what > 0)
					sleep(10)
					thisInterruption.what = thisInterruption.what - 1
					new/obj/effects/crackling(thisInterruption.subject.loc)
					return(1)

			if ("die")
				thisMob.Die() // Purges action queue and whatnot

			/* Wait does not seem to work - second sleep not allowed?
			if ("Wait")
				sleep(thisInterruption.what) 					*/
			else
				DebugMsg("Unhandled Interruption: [thisInterruption.action]")

		if (thisMob.interruptions != null) thisMob.interruptions.Cut(1,2)
		else // Needed to make sure you're not drawing this on a corpse
			if (thisMob.client != null)
				if (istype(thisMob,/mob/player))
					var/mob/player/thisPlayer = thisMob
					thisPlayer.client.AddClientScreenActionAll() // I wonder why I'm calling this here?
		return(1)
	return(0)


