/*
	Project Xenoverse
	By Geldon Yetichsky

file:
ClientScreen.dm

notes:
  Handles client functionality related to manipulating the client.screen variable.
  The unique aspect of the client.screen list is that whatever images() are put on there are uniquely idsplayed to a specific
client.  This makes them useful for functionality related to displaying command queues or goals specific to that PC.
*/

client

	proc/InitializeClientScreenVariables()
		// imgActionQueue = new/list() // This is now stored on the individual Actions in mob.actions.myImages
		/* Well, since I've got nothing else to do, and we are after all initiaizing the PCScreenVariables, lets just
		  clear them all out */

		for (var/image/thisImage in src.images)
			src.images.Remove(thisImage)

	/*
		RedrawClientScreenAll()
		Redraws all screen layers that should be displayed on the client.
		E.g. PCQueuescreenLayers, PCGoalScreenLayers, ect. */
	proc/RedrawClientScreenAll()
		// Removal of all screen layers
		RemoveClientScreenActionAll()

		// Redrawing of all screen layers
		AddClientScreenActionAll()

	/*
		PurgeAllScreenLayers()
		Purges all images from the client.screen variable. */
	proc/PurgeAllScreenLayers()
		while(screen.len > 0)
			screen.Cut(1,2)

	/*
		AddClientScreenActionAll()
		Redraws the kinds of screen objects associated with the PC queue screen layer.
		Naturally, this involves going through a mob/player/ and parsing the action list.
		Note: requires a imgActionQueue list.
	*/
	proc/AddClientScreenActionAll()
		if (mob.actions != null)
			for (var/Action/thisAction in mob.actions)
				AddClientScreenAction(thisAction)

	/*
		AddClientScreenAction()
		Of course, it would be a whole lot more efficient if I only redraw a single action rather than redrawing all
		the actions in the queue every time a new action is entered.  That's what this proc does.
	*/
	proc/AddClientScreenAction(var/Action/thisAction)
		var/image/imagesToUpdate = thisAction.GetClientScreenImages()
		if (imagesToUpdate != null)
			for (var/image/thisImage in imagesToUpdate)
				if (!(thisImage in images)) // Might be laggy, but prevents dupes.
					images.Add(thisImage)
	/*
		RemoveClientScreenActionAll()
		Remove the kinds of screen objects associated with the PC queue screen layer.
		Note: requires a imgActionQueue list.
 */
	proc/RemoveClientScreenActionAll()
		if (mob.actions != null)
			for (var/Action/thisAction in mob.actions)
				RemoveClientScreenAction(thisAction)

	/*
		RemoveClientScreenAction(var/Action/thisAction)
		Like with AddClientScreenAction() this will *Remove* just a single screen
	*/
	proc/RemoveClientScreenAction(var/Action/thisAction)
		// var/imageCount = 0
		if (thisAction.myImages != null)
			for (var/image/thisImage in thisAction.myImages)
				images.Remove(thisImage)
				// imageCount++
		// DebugMsg("RemoveClientScreenAction() executed.  Removed images: [imageCount] ")