/*
	Project Xenoverse
	By Geldon Yetichsky

file:
ClientSkin.dm

notes:
Handles updates and other operations that occur to the 4.0x skins.
*/

var/atom/lastScannerObjectUpdated

client

	proc/SkinResetRespawnScreen()
		winset(src,"childView","left=paneViewRespawn")
		// winset(src,"childSidebar","left=paneSidebarRespawn")

	proc/SkinResetAdventureScreen()
		winset(src,"childView","left=paneViewMap")
		winset(src,"childSidebar","left=paneSidebarUnitControl")
		SkinResetAllStatusInfo()

	proc/SkinResetAllStatusInfo()
		var/mob/player/myPlayer = mob
		winset(src,"lblPlayerName","text=\"[myPlayer.name]\"")
		SkinUpdatePrestige()
		SkinUpdateMass()
		SkinUpdateEnergy()
		UpdateAbilityBank()
		UpdateActiveAbility()
		SkinUpdateScanner()

	proc/SkinUpdatePrestige()
		var/tmp/mob/player/myPlayer = mob
		winset(src,"lblPrestige","text=\"[myPlayer.GetPrestige()]\"")

	proc/SkinUpdateMass()
		if (!istype(mob,/mob/player))
			DebugMsg("SkinUpdateMass() mob was not a player.")
			return()

		var/mob/player/myPlayer = mob
		var/playerMass = myPlayer.GetMass()

		winset(src,"lblCurrentMass","text=\"[myPlayer.mass]\"")

		/* Updating the mass bars takes a bit of extra work because there's 3 bars reflecting levels of encumberance. */

		switch (myPlayer.GetMass())
			if (0 to MAX_MASS_LIGHT_ENCUMBERANCE)
				var/lightEncumberanceValue = playerMass * 100 / MAX_MASS_LIGHT_ENCUMBERANCE
				winset (src,"barLightArmorReadout","value=[lightEncumberanceValue]")
				winset (src,"barMediumArmorReadout","value=0")
				winset (src,"barHeavyArmorReadout","value=0")

			if (MAX_MASS_LIGHT_ENCUMBERANCE to MAX_MASS_MEDIUM_ENCUMBERANCE)
				var/mediumEncumberanceValue = playerMass * 100 / MAX_MASS_MEDIUM_ENCUMBERANCE
				winset (src,"barLightArmorReadout","value=100")
				winset (src,"barMediumArmorReadout","value=[mediumEncumberanceValue]")
				winset (src,"barHeavyArmorReadout","value=0")
			if (MAX_MASS_MEDIUM_ENCUMBERANCE to MAX_MASS_HEAVY_ENCUMBERANCE)
				var/heavyEncumberanceValue = playerMass * 100 / MAX_MASS_HEAVY_ENCUMBERANCE
				winset (src,"barLightArmorReadout","value=100")
				winset (src,"barMediumArmorReadout","value=100")
				winset (src,"barHeavyArmorReadout","value=[heavyEncumberanceValue]")
			else
				DebugMsg("SkinUpdateMass: unhandled mass value: [myPlayer.GetMass()]")

	proc/SkinUpdateEnergy()
		if (mob != null)
			var/mob/player/thisPlayer = mob
			winset(src,"lblCurrentEnergy","text=[thisPlayer.GetEnergy()]")
			var/energyValue = mob.GetEnergy()
			var/maxEnergyValue = mob.GetMaxEnergy()
			var/energyBarValue
			if (maxEnergyValue == 0)
				energyBarValue = 0
			else
				energyBarValue = (energyValue * 100) / maxEnergyValue
			winset(src,"barEnergy","value=[energyBarValue]")

	proc/SkinUpdateNutrition()
		if (mob != null)
			var/mob/player/thisPlayer = mob
			winset(src,"lblCurrentNutrition","text=[thisPlayer.GetNutrition()]")
			var/nutritionValue = mob.GetNutrition()
			var/maxNutritionValue = mob.GetMaxNutrition()
			var/nutritionBarValue
			if (maxNutritionValue == 0)
				nutritionBarValue = 0
			else
				nutritionBarValue = (nutritionValue * 100) / maxNutritionValue
			winset (src,"barNutrition","value=[nutritionBarValue]")

/* UpdateAbilityBank()
	To be executed whenever the player mob's contents reflecting the programs are updated and ready to be displayed
	to the player.  Note that it will take an argument of a specific program within the mob's inventory.  This is useful
	for updating something such as uses without having to redraw the entire grid (which looks ugly). */

	proc/UpdateAbilityBank()

		var/countOfAbilities = 0

		for (var/obj/ability/thisAbility in mob)
			countOfAbilities++

		winset(src, "gridAvailableAbilities", "cells=[countOfAbilities]")  // cut off any remaining cells

		countOfAbilities = 0

		for (var/obj/ability/thisAbility in mob)
			countOfAbilities++
			src << output(thisAbility,"gridAvailableAbilities:[countOfAbilities]")

/*	UpdateActiveAbility()
	  Recreates and redisplays the activeAbilityIcon depending on what the client activeAbility is.
	  Since it is drawn in the same area, the associated skill, skill level, and skill learning cost is included. */
	proc/UpdateActiveAbility()

		if (activeAbility == null)
			activeAbility = new/obj/ability(null)

		if (activeAbility.type == /obj/ability)
			winset (src, "lblActiveAbilityName", "text=")
			winset (src, "lblActiveAbilityDesc", "text=\"No active ability selected.\"")
			// winset (src, "lblActiveAbilityEnergyCost", "text=")

		else

			// These are set on the ability tab pane.
			winset (src, "lblActiveAbilityName", "text=\"[activeAbility.name]\"")
			winset (src, "lblActiveAbilityDesc", "text=\"[activeAbility.desc]\"")
			// winset (src, "lblActiveAbilityEnergyCost", "text=[activeAbility.GetEnergyCost()]")

		winset(src, "gridActiveAbility", "cells=1x1")
		src << output(activeAbility, "gridActiveAbility:1")

		// Setup of the actual active ability on the grid occurs here.  It diverts earlier depending on if it was null or not.

/*  UpdateAbilityHotkey()
	  Recreates and redisplays the F1AbilityIcon - F12AbilityIcon
	  Access an arg which can be a string referring to a hotkey ("F1" - "F12") */

	proc/UpdateAbilityHotkey(var/hotkeyToUpdate)

		if (hotkeyToUpdate == null)
			hotkeyToUpdate = "All"

		if (hotkeyToUpdate == "F1" || hotkeyToUpdate == "All")

			if (F1Ability == null)
				F1Ability = new/obj/ability(null)

			src << output(F1Ability, "gridF1Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F2" || hotkeyToUpdate == "All")
			if (F2Ability == null)
				F2Ability = new/obj/ability(null)

			src << output(F2Ability, "gridF2Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F3" || hotkeyToUpdate == "All")
			if (F3Ability == null)
				F3Ability = new/obj/ability(null)

			src << output(F3Ability, "gridF3Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F4" || hotkeyToUpdate == "All")
			if (F4Ability == null)
				F4Ability = new/obj/ability(null)

			src << output(F4Ability, "gridF4Ability:1,1")

			if (hotkeyToUpdate != "All") return()


		if (hotkeyToUpdate == "F5" || hotkeyToUpdate == "All")
			if (F5Ability == null)
				F5Ability = new/obj/ability(null)

			src << output(F5Ability, "gridF5Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F6" || hotkeyToUpdate == "All")
			if (F5Ability == null)
				F5Ability = new/obj/ability(null)

			src << output(F5Ability, "gridF5Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F7" || hotkeyToUpdate == "All")
			if (F7Ability == null)
				F7Ability = new/obj/ability(null)

			src << output(F7Ability, "gridF7Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F8" || hotkeyToUpdate == "All")
			if (F8Ability == null)
				F8Ability = new/obj/ability(null)

			src << output(F8Ability, "gridF8Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F9" || hotkeyToUpdate == "All")
			if (F9Ability == null)
				F9Ability = new/obj/ability(null)

			src << output(F9Ability, "gridF9Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F10" || hotkeyToUpdate == "All")
			if (F10Ability == null)
				F10Ability = new/obj/ability(null)

			src << output(F10Ability, "gridF10Ability:1,1")

			if (hotkeyToUpdate != "All") return()

		if (hotkeyToUpdate == "F11" || hotkeyToUpdate == "All")
			if (F11Ability == null)
				F11Ability = new/obj/ability(null)

			src << output(F11Ability, "gridF11Ability:1,1")

			if (hotkeyToUpdate != "All") return()


		if (hotkeyToUpdate == "F12" || hotkeyToUpdate == "All")
			if (F12Ability == null)
				F12Ability = new/obj/ability(null)

			src << output(F12Ability, "gridF12Ability:1,1")

			return() // By taking the check off here, the next line works.

		DebugMsg("UpdateAbilityHotkey(): Invalid arg passed : [hotkeyToUpdate]")

	proc/UpdateDisplayPerTurn()
		if (mob.client != null)
			var/mob/player/thisPlayer = mob
			thisPlayer.client.SkinUpdateEnergy()
			thisPlayer.client.SkinUpdateMass()
			// thisPlayer.client.SkinUpdateScanner()

	proc/SkinUpdateScanner(var/atom/objectToScan)

		if (lastScannerObjectUpdated != null && lastScannerObjectUpdated == statobj)
			return()

		var/tmp/mob/player/thisPlayer = mob

		if (statobj == null)
			if (thisPlayer.scanObject != null)
				SkinUpdateScannerNoTarget()
		else
			if (thisPlayer.scanObject == null)
				SkinUpdateScannerNewTarget()
			else
				if (thisPlayer.scanObject == statobj) // Already updated
					SkinUpdateScannerOldTarget()
				else
					SkinUpdateScannerNewTarget()

	proc/SkinUpdateScannerNoTarget()
		var/tmp/mob/player/thisPlayer = mob
		thisPlayer.scanObject = null
		winset(src,"childTargetDisplay","left=paneTargetDisplayNone")

	proc/SkinUpdateScannerOldTarget()

		var/tmp/mob/player/thisPlayer = mob

		if (istype(thisPlayer.scanObject,/obj/device))
			var/obj/device/thisDevice = thisPlayer.scanObject
			var/energyBarValue = round(thisDevice.GetEnergy() * 100 / thisDevice.GetMaxEnergy())
			winset(src,"barScannerDeviceEnergy","value=[energyBarValue]")
		else
			winset(src,"childTargetDisplay","left=paneTargetDisplayNone")

	proc/SkinUpdateScannerNewTarget()

		var/tmp/mob/player/thisPlayer = mob

		thisPlayer.scanObject = statobj // This should only be switch here, since scanObj is used to detect this has been run.

		if (istype(thisPlayer.scanObject,/obj/device))
			var/obj/device/thisDevice = thisPlayer.scanObject
			winset(src,"childTargetDisplay","left=paneTargetDisplayDevice")
			src << output(statobj,"gridScannerDevice:1,1")
			winset(src,"lblScannerDevice","text=\"[thisPlayer.scanObject.name]\"")
			winset(src,"lblScannerDeviceDesc","text=\"[thisPlayer.scanObject.desc]\"")
			var/energyBarValue = round(thisDevice.GetEnergy() * 100 / thisDevice.GetMaxEnergy())
			winset(src,"barScannerDeviceEnergy","value=[energyBarValue]")

		else
			winset(src,"childTargetDisplay","left=paneTargetDisplayNone")

	/* SkinPopulateControlBreakdown()
		This populates the respawn control introduction screen to let the player know what the different keys do */
	proc/SkinPopulateControlBreakdown()

		var/list/obj/ability/abilitiesToAdd = new/list()
		var/list/keysToUseAbility = new/list()

		winset (src,"gridControlBreakdown","cells=0")

		if (Ability_Q != null && Ability_Q.type != /obj/ability)
			abilitiesToAdd.Add(Ability_Q)
			keysToUseAbility.Add("Q or Upper Left Cursor")

		if (Ability_W != null && Ability_W.type != /obj/ability)
			abilitiesToAdd.Add(Ability_W)
			keysToUseAbility.Add("W or Up Cursor")

		if (Ability_E != null && Ability_E.type != /obj/ability)
			abilitiesToAdd.Add(Ability_E)
			keysToUseAbility.Add("E or Upper Right Cursor")

		if (Ability_R != null && Ability_R.type != /obj/ability)
			abilitiesToAdd.Add(Ability_R)
			keysToUseAbility.Add("R or Keypad Minus")

		if (Ability_A != null && Ability_A.type != /obj/ability)
			abilitiesToAdd.Add(Ability_A)
			keysToUseAbility.Add("A or Left Cursor")

		if (Ability_S != null && Ability_S.type != /obj/ability)
			abilitiesToAdd.Add(Ability_S)
			keysToUseAbility.Add("S or Keypad Center")

		if (Ability_D != null && Ability_D.type != /obj/ability)
			keysToUseAbility.Add("D or Right Cursor")

		if (Ability_F != null && Ability_F.type != /obj/ability)
			abilitiesToAdd.Add(Ability_F)
			keysToUseAbility.Add("F or Keypad Plus")

		if (Ability_Z != null && Ability_Z.type != /obj/ability)
			abilitiesToAdd.Add(Ability_Z)
			keysToUseAbility.Add("Z or Lower Left Cursor")

		if (Ability_X != null && Ability_X.type != /obj/ability)
			abilitiesToAdd.Add(Ability_X)
			keysToUseAbility.Add("X or Down Cursor")

		if (Ability_C != null && Ability_C.type != /obj/ability)
			abilitiesToAdd.Add(Ability_C)
			keysToUseAbility.Add("C or Lower Right Cursor")

		if (Ability_V != null && Ability_V.type != /obj/ability)
			abilitiesToAdd.Add(Ability_V)
			keysToUseAbility.Add("V or Keypad Enter")

		if (abilitiesToAdd.len == 0)
			return()
		else
			winset (src,"gridControlBreakdown","cells=[abilitiesToAdd.len]x3")
			var/controlCount = 0
			for (var/obj/ability/thisAbility in abilitiesToAdd)
				controlCount++
				src << output(thisAbility,"gridControlBreakdown:1,[controlCount]")
				src << output(keysToUseAbility[controlCount],"gridControlBreakdown:2,[controlCount]")
				src << output(thisAbility.desc,"gridControlBreakdown:3,[controlCount]")

	proc/GetNullStatusIcon()
		return (GLOBAL_ABILITY_NULL)

	/*
		Updates the screen related to selecting a respawn spot.
	*/
	proc/RepopulateSpawnChamberGrid()
		if (mob == null)
			DebugMsg("RepopulateSpawnChamberGrid() requires a mob.")
			return()
		if (!(istype(mob,/mob/player)))
			DebugMsg("RepopulateSpawnChamberGrid() requires a /mob/player.")
			return()

		var/list/accessibleRespawnChambers = new/list()

		for (var/obj/device/spawnchamber/thisChamber in world)
			if (thisChamber.CanRespawnPlayer(mob))
				accessibleRespawnChambers.Add(thisChamber)

		if (accessibleRespawnChambers.len > 0)
			winset (src,"gridRespawnChambers","cells=3x[accessibleRespawnChambers.len]")

			var/count = 0

			for (var/obj/device/spawnchamber/thisChamber in accessibleRespawnChambers)
				count++
				src << output (thisChamber,"gridRespawnChambers:1,[count]")
				src << output ("[thisChamber.GetSectorName()]:([thisChamber.x],[thisChamber.y])","gridRespawnChambers:2,[count]")
				src << output ("No opportunities detected.","gridRespawnChambers:3,[count]")
		else
			winset (src,"gridRespawnChambers","cells=1x1")
			src << output("We're sorry, there are no accessible materializers at this time.  Please try again later.","gridRespawnChambers")

	/*
		Populates the screen related to purchasing ability program upgrades.
	*/
	proc/SkinPurchaseScreenPopulate()

		winset(src,"gridAbilitiesForSale","cells=citizenAbilityPurchaseList")

		for (var/obj/ability/thisAbility in citizenAbilityPurchaseList)
			src << output(thisAbility,"gridAbilitiesForSale")

		// You're kidding me, that's it?  Almost..

		SkinPurchaseScreenSelectedNewAbility()

	proc/SkinPurchaseScreenSelectedNewAbility()

		if (activeAbility != null && istype(activeAbility,/obj/ability) && activeAbility.type != /obj/ability)
			var/obj/ability/thisAbility = activeAbility

			// DebugMsg("SkinPurchaseSelectedNewAbility() got here.")

			winset(src,"gridPurchaseAbilitySelected","cells=1x1")
			src << output(thisAbility,"gridPurchaseAbilitySelected:1,1")
			winset(src,"lblRespawnEarthIncSelectedAbility","text=\"[thisAbility.name]\"")
			winset(src,"lblRespawnEarthIncSelectedAbilityDesc","text=\"[thisAbility.desc]\"")
			winset(src,"lblRespawnEarthIncSelectedAbilityCostInMass","text=[thisAbility.GetMassCost()]")
			winset(src,"lblRespawnEarthIncSelectedAbilityCostInEnergy","text=[thisAbility.GetEnergyCost()]")
			winset(src,"btnAbilityPurchaseButton","text=\"Purchase this for the low cost of [thisAbility.GetPrice(mob)]!\"")
		else
			winset(src,"gridPurchaseAbilitySelected","cells=0")
			winset(src,"lblRespawnEarthIncSelectedAbility","text=")
			winset(src,"lblRespawnEarthIncSelectedAbilityDesc","text=")
			winset(src,"lblRespawnEarthIncSelectedAbilityCostInMass","text=")
			winset(src,"lblRespawnEarthIncSelectedAbilityCostInEnergy","text=")
			winset(src,"btnAbilityPurchaseButton","text=\"When you have an ability selected, click here to purchase it!\"")

