/*
	Project Xenoverse
	By Geldon Yetichsky

notes:
	General mob handling.  Calls I would like to apply to all mobs should go here.

*/

var/const/RENEGADE_LEVEL_PRESTIGE = -100

mob
	density = 1

	var/const/DEFAULT_MOVEMENT_TIME_IN_MS = 10

	New()

		DefaultNewMobCommoditySetup()

		// This sort of a default icon just so we can see here's a mob that has no icon.
		if (icon == null)
			icon = 'imgControlScheme.dmi'
			icon_state = "whitearrow"
		else
			SouthTo8DirIcon()

		..()

	Click(var/location,var/control,var/params)
		if (istype(usr,/mob/player))
			usr.client.statobj = src
		..()

	DblClick(var/location,var/control,var/params)
		var/tmp/turf/thisTurf = src.loc

		thisTurf.DblClick(location,control,params)
		..()

	/* Basically, does everything that the mob does when it has completed moving. */
	proc/ActionMoved()

		sleep(DEFAULT_MOVEMENT_TIME_IN_MS)

	proc/ActionTurned()

		sleep(DEFAULT_MOVEMENT_TIME_IN_MS/2)

	proc/GetAlignment()
		if (istype(src,/mob/player))
			var/mob/player/thisPlayer = src
			if (thisPlayer.prestige > RENEGADE_LEVEL_PRESTIGE)
				return(CORPORATE_ALIGNMENT)
			else
				return(RENEGADE_ALIGNMENT)
		else
			DebugMsg("[type].GetAlignment() cannot handle type yet.")
			return(WILD_ALIGNMENT)

	/* IsRenegade() and IsCitizen() are currently not coded right.
	   Eventually, these will differ in that they're more along the lines of what you last spawned as, not your
	     current prestige level.*/
	proc/IsRenegade()
		if (GetAlignment() == RENEGADE_ALIGNMENT)
			return(1)
		else
			return(0)

	proc/IsCitizen()
		if (GetAlignment() == CORPORATE_ALIGNMENT)
			return(1)
		else
			return(0)


	/* Animation procs -- Rebuild the mob's icon based off of multiple layers depending on what they have equipped and
	what they do.

	E.g.
		AnimateMovement() -- Animates them walking from space to space
		AnimateIdle() -- Animates them idling, not moving
		AnimateAttack() -- Animates them attacking

		The modifier argument is available in the event that there's different kinds of movement to animate.
	*/

	/* IsAlive()
		Returns 1 if alive, else 0 */
	proc/IsAlive()
		if (actions != null)
			if (src in respawnArea)
				return(0)
			else
				return(1)
		else
			return(0)

	/* Damage()
		Applies damage to mobs.
		Should be defined individually via object orientation because different things react differently to damage */
	proc/Damage(var/Attack/thisAttack)
		switch (thisAttack.GetDamageWord())
			if ("electric")
				Noise(src,ELECTRIC_DAMAGE_SOUND,100)
			else
				Noise (src,pick(ARMOR_RICHOCHET_SOUNDS),100)

		if (mass > rand(1,100))
			F_damage(src, thisAttack.power, "#FF0000")
			mass = mass - thisAttack.power
			if (mass < 0)
				mass = 0
			// These GameOutputs are really ugly ways to let the player know they've been hurt.  Ideally FX will make them redundant.
			GameOutput("[thisAttack.power] units of [thisAttack.GetDamageWord()] damage taken!  Chance of surviving next attack: [mass]%",src)

		else
			GameOutput("The [thisAttack.GetDamageWord()] attack pierces your armor, killing you!",src)
			Noise (src,MOB_PLAYER_DEATH_SOUND,100)
			Die()

	/* Jolt()
		Handles what happens when the mob comes into contact with an energy projectile.
		This is actually fairly universal.  It generates an electrical attack if the energy amount exceeds the amount
		that this mob can store.  */
	proc/Jolt(var/energyJolted)

		var/energyOverMaxEnergy = (GetEnergy() + energyJolted) - GetMaxEnergy()
		if (energyOverMaxEnergy > 0)
			var/Attack/newElectricalAttack = new()
			newElectricalAttack.attackType = ELECTRICAL_ATTACK_TYPE
			newElectricalAttack.power = energyOverMaxEnergy
			Damage(newElectricalAttack)

		/*
		if (istype(src,/mob/alien))
			var/tmp/mob/alien/thisAlien = src
u
			var/magnitude = EnergyToMagnitude(energyJolted)
			thisAlien.NewInterrupt("stun",src,magnitude)
		*/

	/* SouthTo8DirIcon()
		Takes an icon consisting of only one direction (south) and constructs 7 other directions for it,
		  essentially turning it into an 8-direction icon that is natively handled by BYOND.
		  Of course, this is best applied to icons which are displayed in a completely top-down fashion. */

	proc/SouthTo8DirIcon()

		/* I should add some code here to check to see if this particular icon has already been created against a global
		   list of already instantiated icons and, if so, simply return that. */

		var/icon/baseIcon = new()
		var/icon/changeIcon
		var/icon/changeIconDiag

		for (var/thisState in icon_states(icon))

			changeIcon = icon(icon,thisState)
			baseIcon.Insert(changeIcon,icon_state=thisState,dir=SOUTH)
			changeIcon.Turn(90)
			baseIcon.Insert(changeIcon,icon_state=thisState,dir=WEST)
			changeIcon.Turn(90)
			baseIcon.Insert(changeIcon,icon_state=thisState,dir=NORTH)
			changeIcon.Turn(90)
			baseIcon.Insert(changeIcon,icon_state=thisState,dir=EAST)

			/*
			  Some pixelation is inevitable in the case of turning an icon any degree that is not a multiple of 90,
			  but this can be minimized by performing a slight thickening of the icon.  A pixel thicker may not
			  even be noticed by most players' eyes, but if so you could promote uniformity by also thickening the ones above.
			*/

			changeIcon.Turn(90) // Return changeIcon to the default facing of south.

			// Blending together icon with one shifted a pixel out of place.
			changeIcon.Shift(NORTH,1)
			changeIconDiag = icon(icon,thisState)
			changeIconDiag.Blend(changeIcon,ICON_OVERLAY)

			changeIconDiag.Turn(45) // Turn so it's now diagonal, then it's back to business as usual.

			baseIcon.Insert(changeIconDiag,icon_state=thisState,dir=SOUTHWEST)
			changeIconDiag.Turn(90)
			baseIcon.Insert(changeIconDiag,icon_state=thisState,dir=NORTHWEST)
			changeIconDiag.Turn(90)
			baseIcon.Insert(changeIconDiag,icon_state=thisState,dir=NORTHEAST)
			changeIconDiag.Turn(90)
			baseIcon.Insert(changeIconDiag,icon_state=thisState,dir=SOUTHEAST)

		icon = baseIcon

	/*
		GetSoundEnvironment()
		This is used for setting sound.environment

		In the future, it may be useful to actually evaluate the environment the player is in.
		For now, we're just going to assume we're always in a specific type of room.
	*/

	proc/GetSoundEnvironment()

		return(12) // 12 = Hallway.  See sound.environment help for specific environment types.

		/* GameOutput()
			Spams a text string to the outputGame control  */
	proc/GameOutput(var/textToSpam)
		if (client != null)
			src << output("[textToSpam]","outputGame")