/*
	Project Xenoverse
	By Geldon Yetichsky

notes:

Having had it revealed that the BYOND engine is really more comfortable with there being only one player mob that isn't
swapped between, I'm thinking I'm going to make this very much a single-character-at-a-time RPG once again.  However, I'm
integrating a great deal of engine friendliness while maintaining both a higher-than-usual degree of interactivity and
dynamic content by making the player character a vessel for a highly interactive/dynamic skill system that grants them
a wide variety of influential moves.

Our player mob is now a sort of intergalactic troubleshooter who can whip solutions out of nowhere.  This works hand-in-hand
with our virtual GM.

*/

// var/const/DEATH_MESSAGE_BEFORE_RESPAWN = "Your body collapses in a dying heap beneath you."
var/const/MOB_PLAYER_DEATH_SOUND = 'Mechanic-Public_D-139.wav'

	// Maximum number of armor points
var/const/MAX_MASS_LIGHT_ENCUMBERANCE = 100
var/const/MAX_MASS_MEDIUM_ENCUMBERANCE = 200
var/const/MAX_MASS_HEAVY_ENCUMBERANCE = 300

	// Time (in ms) to move depending on the player's current armor level.
var/const/LIGHT_ENCUMBERANCE_DELAY = 10
var/const/MEDIUM_ENCUMBERANCE_DELAY = 15
var/const/HEAVY_ENCUMBERANCE_DELAY = 20

	// Citizen Initialization Variables
var/const/NEW_CITIZEN_PRESTIGE_AMOUNT = 100
var/const/CITIZEN_DEFAULT_ARMOR_VALUE = 100
var/const/CITIZEN_MASS_START_ON_RESPAWN = 100
var/const/CITIZEN_ENERGY_START_ON_RESPAWN = 100
var/const/CITIZEN_NUTRITION_START_ON_RESPAWN = 100
var/const/CITIZEN_ENERGY_REGENERATED_PER_TURN = 5
var/const/CITIZEN_AUTOMATIC_NUTRITION_BOOST_LEVEL = 25
var/const/CITIZEN_NUTRITION_BOOST_PRESTIGE_COST_PER_ABILITY = 1
var/const/NUTRITION_CONSUMED_PER_TURN = 0.1

	// Renegade Initializaiton Variables
var/const/RENEGADE_MASS_START_ON_RESPAWN = 0
var/const/RENEGADE_ENERGY_START_ON_RESPAWN = 0
var/const/RENEGADE_NUTRITION_START_ON_RESPAWN = 25 // In minutes?

var/const/MOB_PLAYER_DEFAULT_MAX_ENERGY = 250 // In the future, I might decide to make this an adjustable value.

mob/player

	var prestige // Prestige, used specifically for score determining and whether or not you have entered renegade status
	var/atom/scanObject // What'll be scanned when it hits 0.

	/* Related to flashlight functionality: The players by default cast light and here is an object to store their
	  flashlight. */
	luminosity = 1
	var/list/obj/effects/light/flashlight

	icon = 'imgPlayer.dmi'
	icon_state = "idle"

	New()

		/* Before we move the player into the respawn area, we need to make sure they are created first. */
		spawn()
			InitializeAsNewPlayer()

		..()

	/*
		Login()
		Note that this is what's used instead of New() if the player/mob has already been created.
	*/
	Login()
		client.SetKeypadToMovement()
		if (src in respawnArea)
			client.SkinResetRespawnScreen()
		else
			client.SkinResetAdventureScreen()

		GrantGmVerbs()

		..()

	/*
		Logout()
		Note for something to do later: make it so the PC will attempt to automatically walk back to the nearest
		allied respawn portal in the event that the player has logged out for more than 5 minutes.
	*/

	/* Die()
	  If Object Orientation is as good as I think it is, this should fall back naturally to the Die() proc
	  in the ActionQueue file (so long as I include ..()) and I just resolve mob/player specifics here. */
	Die()

			// If actions are already null, you're already dead.
		if (actions != null)
			// Fancy death FX go here!
			src.Move(locate(/area/respawn))

			// This is probably an error state.
		else if (!(src in respawnArea))
			src.Move(locate(/area/respawn))

		..() // This is supposed to fall to /mob.Die() which deletes actions.

	/* Action() procs are called by the ActionQueue to reflect affects unique to a given mob.
	   At this point, the action has already occured, these handle the fallout. */

	ActionMoved()

		ShineLight()

		switch (mass)
			if (0 to 100)
				sleep(LIGHT_ENCUMBERANCE_DELAY)
			if (101 to 200)
				sleep(MEDIUM_ENCUMBERANCE_DELAY)
			else
				sleep(HEAVY_ENCUMBERANCE_DELAY)

	ActionTurned()

		ShineLight()

		switch (mass)
			if (0 to 100)
				sleep(round(LIGHT_ENCUMBERANCE_DELAY/2))
			if (101 to 200)
				sleep(round(MEDIUM_ENCUMBERANCE_DELAY/2))
			else
				sleep(round(HEAVY_ENCUMBERANCE_DELAY/2))


	proc/ActionRespawned()

		if (GetAlignment() == CORPORATE_ALIGNMENT)
			SetMass(CITIZEN_MASS_START_ON_RESPAWN)
			SetEnergy(CITIZEN_ENERGY_START_ON_RESPAWN)
			SetNutrition(CITIZEN_NUTRITION_START_ON_RESPAWN)
		else
			SetMass(RENEGADE_MASS_START_ON_RESPAWN)
			SetEnergy(RENEGADE_ENERGY_START_ON_RESPAWN)
			SetNutrition(RENEGADE_NUTRITION_START_ON_RESPAWN)

		// This is necessary for both factions to clear out any old light variables.
		ShineLight()

		if (client != null)
			client.SkinResetAdventureScreen()

		if (!IsAlive())
			Live()

	/*
	Damage(var/Attack/thisAttack)
		if (mass > rand(1,100))
			s_damage(src, thisAttack.power, "#FF0000")
			mass = mass - thisAttack.power
			if (mass < 0)
				mass = 0
			// These GameOutputs are really ugly ways to let the player know they've been hurt.  Ideally FX will make them redundant.
			GameOutput("[thisAttack.power] units of [thisAttack.GetDamageWord()] damage taken!  Chance of surviving next attack: [mass]%",src)
			Noise (src,pick(ARMOR_RICHOCHET_SOUNDS),100)
		else
			GameOutput("The [thisAttack.GetDamageWord()] attack pierces your armor, killing you!",src)
			Noise (src,MOB_PLAYER_DEATH_SOUND,100)
			Die()
	*/

	proc/HasGMAccess()
		if (client.key == null)
			return(0)

		if (client.key == world.host)
			return(1)

	/*
		InitializeAsNewPlayer()
		This basically resets a player from scratch.  It's the proc to call when a brand new player enters the game
		or an old player needs to be reinitialized (perhaps the game has reset).
	*/

	proc/InitializeAsNewPlayer()

		prestige = NEW_CITIZEN_PRESTIGE_AMOUNT
		maxEnergy = MOB_PLAYER_DEFAULT_MAX_ENERGY

		if (client != null)
			client.SetKeypadToMovement()

		for (var/obj/ability/thisAbility in src.contents)
			del thisAbility

		/* Assignment of default abilities that players get
		new/obj/ability/matterconverter(src)
		new/obj/ability/energyconverter(src)
		new/obj/ability/discharge(src)
		*/

		Move(respawnArea)

	proc/ShineLight() // Deletes contents of flashlight list, then creates a new batch based off of what the player can see.

		// First, we purge the old list

		if (flashlight == null)
			flashlight = new/list()
		else
			while (flashlight.len > 0)
				var/tmp/obj/effects/light/thisLight = flashlight[1]
				thisLight.Kill() // Not taking any chances, having the light object destroy itself from the inside.
				flashlight.Cut (1,2)

		if (!IsAlive())
			return()

		if (IsRenegade())
			luminosity = 0
			return()
		else
			luminosity = 1

		// Next, we draw a line up to 5 spaces from the mob.

		var/tmp/turf/destinationTurf
		destinationTurf = get_step (src,dir)

		if (destinationTurf == null) // Should only happen if you're up against the map edge.
			return() // No flashlight for you, then!

		for (var/increment = 0, increment < 5, increment++)
			var/tmp/newTurf = get_step (destinationTurf,dir)
			if (newTurf != null) destinationTurf = newTurf

		var/tmp/list/turf/flashLine = getline(src,destinationTurf)

		// Now, draw a light in every visible hex between you and the destination turf, increasing aperature as we go.

		var/tmp/aperature = 2

		for (var/turf/thisTurf in flashLine)
			if (thisTurf in view(src))
				var/obj/effects/light/newLight = new/obj/effects/light(thisTurf,aperature)
				flashlight.Add(newLight)
			aperature = aperature + 0.5

	/* AdjustPrestige()
		This should be the only way in which Prestige is altered for security reasons and also because eventually I should
		provide a log of all the various ways the player has lost or earned prestige. */
	proc/AdjustPrestige(var/amount,var/reason)

		if (amount == 0)
			return()

		if (prestige == null) // Should not happen if InitializeNewPlayer() has done its job.
			prestige = 0

		if (isnum(amount))
			prestige = prestige + amount
			winset(client,"lblPrestige","text=[GetPrestige()]")
			if (amount > 0)
				if (reason == null)
					GameOutput("[prestige]: Gained for mysterious reasons.")
				else
					GameOutput("You have earned [amount] for [reason].")
			else if (amount < 0)
				if (reason == null)
					GameOutput("[prestige]: Lost for mysterious reasons.")
				else
					GameOutput("You have lost [amount] for [reason].")

		client.SkinUpdatePrestige()

	/* GetPrestige()
		Clearly, the cleanest way to look up your current prestige level. */
	proc/GetPrestige()
		if (prestige == null)
			prestige = 0
		return(prestige)

	/*
	/* Walk()
		This goes through the actionQueue and finds the last movement made, then appends a new move there.
		Not used in driving movement. */

	proc/Walk(var/thisDirection)
		if (!(IsAlive()))
			DebugMsg("Walk(): User attempting to move, but they're dead.  Tough luck!")

		var/tmp/turf/lastStep = null
		var/tmp/turf/newStep = null

		var/tmp/Action/actionExamined

		for (var/thisIncrement = actions.len, thisIncrement > 0, thisIncrement--)
			actionExamined = actions[thisIncrement]
			if (actionExamined.action == "move")
				lastStep = actionExamined.subject
				thisIncrement = 0

		if (lastStep == null) // First step taken
			newStep = get_step(loc,thisDirection)
		else
			newStep = get_step(lastStep,thisDirection)

		if (newStep == null)
			DebugMsg("Walk: User attempting impossible move.")
			return(0)
		// Create a new action and append it to the list.

		NewAction("move",newStep)
	*/

	/* ExecuteSuitGeneratorIteration()
		This is called constantly by the ActionQueue and should check to make sure it's the right time to execute. */
	proc/ExecuteSuitGeneratorIteration()

		AdjustNutrition(-NUTRITION_CONSUMED_PER_TURN)

		if (IsAlive())
			if (IsCitizen())
				if (GetMass() < CITIZEN_DEFAULT_ARMOR_VALUE)
					AdjustMass(1)
				else
					AdjustEnergy(CITIZEN_ENERGY_REGENERATED_PER_TURN)
				if (GetNutrition() < CITIZEN_AUTOMATIC_NUTRITION_BOOST_LEVEL)
					IssueCitizenNutritionBoost()

			// Renegades don't regenerate anything, unlucky sobs.

		// Now we're going to deal with nutrition.


	proc/IssueCitizenNutritionBoost()

		SetNutrition(CITIZEN_NUTRITION_START_ON_RESPAWN)

		var/numberOfAbilities = 1

		for (var/obj/ability/thisAbility in contents)
			numberOfAbilities++

		AdjustPrestige(-(numberOfAbilities * CITIZEN_NUTRITION_BOOST_PRESTIGE_COST_PER_ABILITY),"New You (C) nutrition injection.")

	proc/GrantGmVerbs()

		// Before we give the player GM access, they need to be created frist.
		if (HasGMAccess())
			/* This is if I created a mob called GMVerbs whih had such verbs.
			var/tmp/mob/GMVerbs/thisGM = new()

			for (var/verb/thisVerb in thisGM.verbs)
				src.verbs += thisVerb
			*/

			if (!(/proc/GMDumpVars in client.verbs))
				client.verbs += /proc/GMDumpVars // This is support for Hobnob's DumpVars
			// Any additional GM verbs.

mob/player/DM

	key = "Geldonyetich"

	New()
		..()

	verb/GMTeleport(var/gotoHere as text)
		if (!debugText)
			usr << "Only works in debugging mode."
			usr.loc = locate(gotoHere)


	/* GrantNewAbilities()
		This proc should run through all the skills the player character has and verify that each granted ability type is
		already in the player character's inventory.  If it is not, then it adds it.   Should be run once whenever a
		player has received a new ability (e.g. a skill upgrade).

		Cool code, but ultimately gutted when I decided to move skills to unlocking facilities. */
	/*
	proc/GrantNewAbilities()

		var/list/typeToCreateList = new()

		if (skills.skillConstruction > -1)
			typeToCreateList.Add(/obj/ability/repair)
		if (skills.skillConstruction > 0)
			typeToCreateList.Add(/obj/ability/createfloor)
		if (skills.skillConstruction > 1)
			typeToCreateList.Add(/obj/ability/createwall)
		if (skills.skillConstruction > 2)
			typeToCreateList.Add(/obj/ability/shieldgen)

		DebugMsg("typeToCreateList.len: [typeToCreateList.len]")

		var/matchFound
		for (var/typeToCreate in typeToCreateList)
			matchFound = 0
			for (var/obj/ability/thisAbility in contents)
				if (thisAbility.type == typeToCreate)
					matchFound = 1
			if (matchFound == 0)
				new typeToCreate (src)

		client.UpdateAbilityBank()
		del typeToCreateList
		// DebugMsg("GrantNewAbilities: contents.len: [contents.len]")
	*/
