/*
	Project Xenoverse
	By Geldon Yetichsky

file:
Obj.dm

notes:
	This basically handles all changes done to the base /obj datum.
*/

obj

	/*  mapicons are displayed on client.gridMap.  Best case scenario, these are on a global variable and update automatically. */

	mapicon

		name = ""
		desc = ""

		New(var/location,var/icon/myIcon)
			if (location != null)
				DebugMsg("[type].New() requires null location.")
				spawn()
					del src

			if (!istype(myIcon,/icon))
				DebugMsg("[type].New() requires an icon as second arg.")
				spawn()
					del src
			else
				icon = myIcon

	/*  The TurfDamage object is basically a way of adding damage to a turf without using turf variables.
	    It doubles as a handy damage overlay.  It deletes itself when the damage has been repaired.

	    Current implementation concept for turfDamage is that damageSustained goes from 1 to 5 reflecting how close
	    the turf is to destruction.  It draws a wallcrack appropriate to its damage level.  If the damage falls below one,
	    turfDamage is removed.  If the damage goes above 6, the turf is destroyed completely and turfDamage is no longer
	    necessary.

	    When a turf is damaged, it generates a turfdamage object if the damage is deemed lasting.
	    */

	turfdamage
		icon = 'imgEffects.dmi'

		mouse_opacity = 0
		density = 0
		opacity = 0

		var/damageSustained // Damage in points

		New(var/location,var/firstDamage)

			if (!(istype(location,/turf)))
				DebugMsg("[type] must be created on turfs!")
				del src

			if (firstDamage != null)
				damageSustained = firstDamage

		/* turfDamage.Repair()
			Initially, Repair takes an amtToRepair (numeric value) reflected by the repair ability and makes this a number
			of cycles of damage repaired. */
		proc/Repair(var/amtToRepair)

			if (amtToRepair == null)
				amtToRepair = 1

			damageSustained = damageSustained - amtToRepair
			if (damageSustained < 1)
				del src
			else
				DrawDamage()

		proc/DrawDamage()
			switch (damageSustained)
				if (1)
					icon_state = "wallcrack"
				if (2)
					icon_state = "wallcrack2"
				if (3)
					icon_state = "wallcrack3"
				if (4)
					icon_state = "wallcrack4"
				if (5)
					icon_state = "wallcrack5"
				else
					DebugMsg("Unknown damageSustained amount: [damageSustained]")

		proc/AddDamage(var/amtToAdd)
			damageSustained = damageSustained + amtToAdd

			if (damageSustained > 5)
				var/tmp/turf/thisTurf = loc
				thisTurf.DestroyTurf()
				del src
			else
				DrawDamage()

