/*AbyssDragon.BasicMath v1.1
  written by AbyssDragon (abyssdragon@,molotov.nu)
  http://www.molotov.nu

  (used to be AbyssDragon.AbyssLibrary)

  Feel free to modify/improve/destroy/steal/use any of this code however you see fit.
  A thanks or mention in your project would be nice, but neither are required.

  This is a library of procs to do some basic math and geometrical things.
  Many of the procs in the old version of this have been invalidated since then and have been removed.
  A lot of these procs may be duplicated in other BYOND libraries/demos.  However, all of this code
  was written by me, or was adapted from C libraries/tutorials by me.

  NOTE:  although most of the procs use atoms, you probably shouldn't use areas.
*/

/*
Mathematical Constants:
   pi - should need no explanation
   sqrt2 - the square root of two.  The diaganal distance between two squares.
   e - the base for natural logarithms (and a whole lot more). */
var
	const//Some nice and lengthy mathematical constants.  BYOND rounds them off, but I like to have 'em handy anyway.
		pi =	3.1415926535897932384626433832795
		sqrt2 =	1.4142135623730950488016887242097	//Square root of 2
		e =		2.7182818284590452353602874713527


proc
	/* xrange() takes an atom and a number as arguments, returns all things within that range of the atom,
	  even at ranges greater than 2*world.view */
	xrange(atom/center, range)
		var/startx = center.x - range
		var/starty = center.y - range
		var/endx = center.x + range
		var/endy = center.y + range
		if(startx < 1) startx = 1
		if(starty < 1) starty = 1
		if(endx > world.maxx) endx = world.maxx
		if(endy > world.maxy) endy=world.maxy
		var/contents[] = list()
		for(var/turf/T in block(locate(startx, starty, center.z), locate(endx, endy, center.z)))
			contents += T
			contents += T.contents
		return contents

	/* get_steps() takes an atom, a direction, and a number as an argument, returns a turf that number steps
	  in that direction from the atom.  Essentially a multi-step version get_step */
	get_steps(atom/ref,dir,num)
		var/x
		var/turf/T=ref:loc
		if(isturf(ref))
			T=ref
		for(x=0;x<num;x++)
			ref=get_step(ref,dir)
			if(!ref)break
			T=ref
		return T

	/* allclear() takes a turf as an argument, returns 1 if it is non-dense and contains no dense objects, 0 otherwise */
	allclear(turf/T)
		if(isturf(T))
			if(T.density) return 0
		var/mob/M
		for(M as mob|obj in T)
			if(M.density)
				return 0
		return 1

	/* cardinal() takes an atom as an argument, returns all turfs that are in the 4 cardinal directions from it */
	cardinal(atom/ref)
		return (list(get_step(ref,NORTH),get_step(ref,SOUTH),get_step(ref,EAST),get_step(ref,WEST)))

	/* cardinal_stuff() works just like cardinal, but returns only the contents of those turfs instead */
	cardinal_stuff(atom/ref)
		var/turfs[]=list(get_step(ref,NORTH),get_step(ref,SOUTH),get_step(ref,EAST),get_step(ref,WEST))
		var/stuff[]=list()
		var/turf/T
		for(T in turfs)
			stuff+=T.contents
		return stuff

	// midpoint() takes two atoms as arguments, returns the turf that is the midpoint of the two
	midpoint(atom/M,atom/N)
		var/turf/T = locate((N.x + M.x)/2, (N.y + M.y)/2, (N.z + M.z)/2)
		return T

	// distance() tajes two atoms as arguments, returns the actual distance between the two, not number of steps
	distance(atom/M,atom/N)
		return sqrt((N.x-M.x)**2 + (N.y-M.y)**2)


	// getring() takes an atom and a radius argument, returns a list of turfs forming a ring around the atom at radius distance
	getring(atom/M, radius)
		var/ring[] = list()
		var/turf/T
		for(T as turf in range(radius+1,M))
			if(abs(distance(T, M)-radius) < 0.5) //The < 0.5 check is to make sure the ring is smooth
				ring += T
		return ring

	// getcircle() takes an atom and a radius argument, returns a list of turfs in a circle around the atom within radius distance
	getcircle(atom/M, radius)
		var/list/circle = list()
		var/turf/T
		for(T as turf in range(radius+3,M))		//The < 0.5 check is to ensure it has the same shape as
			if(distance(T, M) < radius + 0.5) 	//a get_ring() of the same radius
				circle += T
		return circle

	/*   sign() takes a number as an argument, returns 1 if the number is positive, -1 if negative, 0 if zero.
	     Adapted from a C tutorial.  Should get bonus points for being the most compact code in the world! */
	sign(x)
		return ((x<0)?-1:((x>0)?1:0))

	/*   getline() takes two atoms as arguments, returns a list of all turfs in a line between them.
	  Uses the well-known Bresenham line-drawing algorithm.  Adapted from a C tutorial. */
	getline(atom/M,atom/N)//Ultra-Fast Bresenham Line-Drawing Algorithm
		var/px=M.x		//starting x
		var/py=M.y
		var/line[] = list(locate(px,py,M.z))
		var/dx=N.x-px	//x distance
		var/dy=N.y-py
		var/dxabs=abs(dx)//Absolute value of x distance
		var/dyabs=abs(dy)
		var/sdx=sign(dx)	//Sign of x distance (+ or -)
		var/sdy=sign(dy)
		var/x=dxabs>>1	//Counters for steps taken, setting to distance/2
		var/y=dyabs>>1	//Bit-shifting makes me l33t.  It also makes getline() unnessecarrily fast.
		var/j			//Generic integer for counting
		if(dxabs>=dyabs)	//x distance is greater than y
			for(j=0;j<dxabs;j++)//It'll take dxabs steps to get there
				y+=dyabs
				if(y>=dxabs)	//Every dyabs steps, step once in y direction
					y-=dxabs
					py+=sdy
				px+=sdx		//Step on in x direction
				line+=locate(px,py,M.z)//Add the turf to the list
		else
			for(j=0;j<dyabs;j++)
				x+=dxabs
				if(x>=dyabs)
					x-=dyabs
					px+=sdx
				py+=sdy
				line+=locate(px,py,M.z)
		return line