/*
	Sectors are specific areas in the game.  Sectors have an alignment which can be flipped by hacking all of the consoles
	in the sector.  Sectors also run the Heartbeat() procs of everything in their area.   This allows them to feature
	different scales of time.  Eventually, sectors will dynamically generate themselves.
*/

var/const/WILD_ALIGNMENT = "Wild"
var/const/RENEGADE_ALIGNMENT = "Renegade"
var/const/CORPORATE_ALIGNMENT = "Citizen"
var/const/UNDEFINED_ALIGNMENT = "UNDEFINED"
var/const/DEFAULT_MS_PER_TURN = 10

Sector

	var/name
	var/alignment
	var/area/sector/myArea
	var/list/obj/device/devicesManaged

	New(var/newAlignment,var/newName,var/area/sector/newArea)

		if (newAlignment != null)
			alignment = newAlignment
		else
			DebugMsg("[type].New() alignment required for arg 1.")
			del src

		if (newName != null)
			name = newName
		else
			DebugMsg("[type].New() name required for arg 2.")
			del src

		// In the future, this would prompt me to create a new swapmap.
		if (isnull(newArea))
			DebugMsg("[type].New() requires newArea.")
			del src
		else
			myArea = newArea
			/*
			for (var/turf/thisTurf in newTurfs)
				myArea.contents.Add(thisTurf)
			*/
			myArea.mySector = src

		msPerTurn = DEFAULT_MS_PER_TURN

		SectorProcess()

area/sector

		// This greatly eases the speed of finding the corresponding sector datum
	var/Sector/mySector

	Entered(var/atom/thisEntered)
		if (istype(thisEntered,/mob/player))
			var/mob/player/thisPlayer = thisEntered
			if (thisPlayer.IsAlive())
				thisPlayer.GameOutput("You have entered the [lowertext(mySector.alignment)]-owned sector [mySector.name].")
		..()


atom/movable

		// This greatly eases the speed of finding the corresponding sector datum
	var/Sector/mySector

	proc/GetSectorName()
		if (isnull (mySector))
			return("Unknown")
		else
			return(mySector.name)

	/*
		The SectorProcess() runs constantly, executing the Hearbeat proc of all appropriate datums at a rate reflecting
		the msPerTurn variable on the sector.
	*/
Sector


	var/msPerTurn
	var/tmp/sectorProcessAlreadyRunning

	proc/SectorProcess()

		if (sectorProcessAlreadyRunning != null)
			return()

		sectorProcessAlreadyRunning = 1

		DebugMsg("Starting sector process...")

		devicesManaged = new/list()

		for (var/obj/device/thisDevice in myArea.contents)
			devicesManaged.Add(thisDevice)
			thisDevice.mySector = src

		spawn()

			while(world.IsRunning())

				for (var/obj/device/thisDevice in devicesManaged)
					// DebugMsg("Running heartbeat on [thisDevice]")
					thisDevice.Heartbeat()

				sleep(msPerTurn)

				// DebugMsg("Reached the end of the list.")

			del sectorProcessAlreadyRunning