/*
	Project Xenoverse
	By Geldon Yetichsky

file:
TurfWall.dm

notes:
	Personal maps are intended to be navigated by player characters.  Their turfs are sorted into being either floors or walls.
	Floors are essentially the ground and generally do not offer obstruction when passed.
	Walls are essentialls walls and usually offer obstruction to passage and sometimes obstruction to sight.

	Personal turfs are completely destructable, although ones that are part of active facilities will likely be rebuilt when
	damaged.

	The new character ship is primarily protected by not allowing anyone armed with anything that can destroy it from entering.
*/

turf/wall

	density = 1
	opacity = 1
	icon = 'TurfsWall.dmi'

	soil
		name = "Bare soil wall"
		desc = "This wall is made up of a compacted stack of low-density rock pebbles."
		icon_state = "soil"

	plexweave
		name = "Plexweave wall"
		desc = "This wall is made of plexweave, a transparent but comparatively frail synthetic solid material."
		icon_state = "plexweave"

		opacity = 0

		New()
			if (underlays.len == 0)
				underlays+= icon('TurfsFloor.dmi',"soil",NORTH)

	alchelite
		name = "Alchelite wall"
		desc = "This wall is made of alchelite, a reflective and moderately solid synthetic material."
		icon_state = "alchelite"

	subneutronium
		name = "Subneutronium wall"
		desc = "This wall is made of subneutronium, an incredibly dense solid synthetic material."
		icon_state = "subneutronium"

	transmatrix
		name = "Transmatrix wall"
		desc = "This wall is made of transmatrix, a completely unbreakable multidimensional anomaly."
		icon_state = "transmatrix"

		transmatrixtransparent
			opacity = 0
			name = "Transmatrix window"
			desc = "This wall is transparent, but still as unbreakably strong as a transmatrix wall."
			icon_state = "transmatrixwindow"

		/* Consoles are intended to be hacked, which flips the ownership of the sector. */
		transmatrixconsole

			name = "Transmatrix console"
			desc = "This transmatrix wall has a console built into it.  The text scrolling across it gives some hint as to who owns it."

			icon_state = "transmatrixconsoleneutral" // This is for map building purposes

			proc/SetupConsoleIconState(var/Sector/mySector)
				switch (mySector.alignment)
					if (WILD_ALIGNMENT)
						icon_state = "transmatrixconsolealien"
					if (RENEGADE_ALIGNMENT)
						icon_state = "transmatrixconsolehostile"
					if (CORPORATE_ALIGNMENT)
						icon_state = "transmatrixconsolefriendly"
					else
						icon_state = "transmatrixconsoleneutral"

			proc/HackedByFriendly()
				SetupConsoleIconState()

			proc/HackedByHostile()
				SetupConsoleIconState()

			proc/HackedByAlien()
				SetupConsoleIconState()







