/*
	Project Xenoverse
	By Geldon Yetichsky

file:
World.dm

notes:
Handles basic world definition and also global variables.

Maybe I should move global variables to a more appropriate place.  It's not like they couldn't exist at the root variable of
any and all files.
*/

#define FILE_DIR icons
#define FILE_DIR sounds

// #include "mapname.dmp"  -- This is how you load additional maps.  They end up on successive z coords.

var/const/debugText = 1  // If true, certain interfaces are made available to facilitate debugging.

var/area/respawn/respawnArea // if nothing is pointing to it, it tends to cease to exist.
var/list/Sector/worldSectors

/* Some day, perhaps.
var/list/swapmap/worldmaps
var/const/DYN_MAPSIZE_X = 500 // Size of dynamic maps, x coord.
var/const/DYN_MAPSIZE_Y = 500 // Size of dynamic maps, y coord.
*/

var/obj/ability/GLOBAL_ABILITY_STEP_FORWARD
var/obj/ability/GLOBAL_ABILITY_STEP_BACKWARDS
var/obj/ability/GLOBAL_ABILITY_TURN_RIGHT
var/obj/ability/GLOBAL_ABILITY_TURN_LEFT
var/obj/ability/GLOBAL_ABILITY_SIDESTEP_RIGHT
var/obj/ability/GLOBAL_ABILITY_SIDESTEP_LEFT
var/obj/ability/GLOBAL_ABILITY_CANCEL_ACTIONS
var/obj/ability/GLOBAL_ABILITY_NULL

var/icon/GLOBAL_SENSOR_LIT_OVERLAY

// Abilities which can be purchased by citizens.
var/list/citizenAbilityPurchaseList

var/icon/GLOBAL_ICON_F1_OVERLAY
var/icon/GLOBAL_ICON_F2_OVERLAY
var/icon/GLOBAL_ICON_F3_OVERLAY
var/icon/GLOBAL_ICON_F4_OVERLAY
var/icon/GLOBAL_ICON_F5_OVERLAY
var/icon/GLOBAL_ICON_F6_OVERLAY
var/icon/GLOBAL_ICON_F7_OVERLAY
var/icon/GLOBAL_ICON_F8_OVERLAY
var/icon/GLOBAL_ICON_F9_OVERLAY
var/icon/GLOBAL_ICON_F10_OVERLAY
var/icon/GLOBAL_ICON_F11_OVERLAY
var/icon/GLOBAL_ICON_F12_OVERLAY

var/list/ARMOR_RICHOCHET_SOUNDS = list('Ground_R-Diode111-8874.wav','Ground_R-Diode111-8879.wav','Ground_R-Diode111-8880.wav','Ground_R-Diode111-8881.wav')
var/ELECTRIC_DAMAGE_SOUND = 'electricshock.wav'

world
	/********* DEFAULTS.  These are all the default world configuration variables.  Very powerful, not to be taken lightly. */

	mob = /mob/player // This is the type of mob created for new players when they log in.
	turf = /turf/wall/soil // This is the default type of turf when none is selected.  It is supposed to automatically mute.

	view = 7  // Determines the size of the players' view window.

	New()	/********** NEW WORLD CREATION. All major global variable declaration should go in here. ***************/

		GLOBAL_ABILITY_STEP_FORWARD = new/obj/ability/stepforward(null)
		GLOBAL_ABILITY_STEP_BACKWARDS = new/obj/ability/stepbackward(null)
		GLOBAL_ABILITY_TURN_RIGHT = new/obj/ability/turnright(null)
		GLOBAL_ABILITY_TURN_LEFT = new/obj/ability/turnleft(null)
		GLOBAL_ABILITY_SIDESTEP_RIGHT = new/obj/ability/stepright(null)
		GLOBAL_ABILITY_SIDESTEP_LEFT = new/obj/ability/stepleft(null)
		GLOBAL_ABILITY_CANCEL_ACTIONS = new/obj/ability/cancelactions(null)
		GLOBAL_ABILITY_NULL = new/obj/ability(null)

		citizenAbilityPurchaseList = new/list()

		citizenAbilityPurchaseList.Add(new/obj/ability/repair(null))

		GLOBAL_ICON_F1_OVERLAY = icon('imgAbilities.dmi',"F1")
		GLOBAL_ICON_F2_OVERLAY = icon('imgAbilities.dmi',"F2")
		GLOBAL_ICON_F3_OVERLAY = icon('imgAbilities.dmi',"F3")
		GLOBAL_ICON_F4_OVERLAY = icon('imgAbilities.dmi',"F4")
		GLOBAL_ICON_F5_OVERLAY = icon('imgAbilities.dmi',"F5")
		GLOBAL_ICON_F6_OVERLAY = icon('imgAbilities.dmi',"F6")
		GLOBAL_ICON_F7_OVERLAY = icon('imgAbilities.dmi',"F7")
		GLOBAL_ICON_F8_OVERLAY = icon('imgAbilities.dmi',"F8")
		GLOBAL_ICON_F9_OVERLAY = icon('imgAbilities.dmi',"F9")
		GLOBAL_ICON_F10_OVERLAY = icon('imgAbilities.dmi',"F10")
		GLOBAL_ICON_F11_OVERLAY = icon('imgAbilities.dmi',"F11")
		GLOBAL_ICON_F12_OVERLAY = icon('imgAbilities.dmi',"F12")

		GLOBAL_SENSOR_LIT_OVERLAY = icon('imgDevices.dmi',"lightsource")

		/*
		spawn()
			HiveMind()
		*/

		respawnArea = locate(/area/respawn)

		spawn()
			sleep(10)
			DebugMsg("Generating world...")

		GenerateNewWorld()

		spawn()
			sleep(10)
			DebugMsg("World generated.")

		BeautificationDaemon()

		spawn()
			sleep(10)
			DebugMsg("BeautificationDaemon ran.")

		..() // Of course, anything the world normally initializes should be done as well.

	/* IsRunning()
		Used by TimeGoblins to determine if they should be running. */
	proc/IsRunning()
		return(1)

	/* GenerateNewWorld()
		Somewhat a misnomer right now, what it's currently doing is just cutting up the existing map into sector
		datums.  In the future, sector datums will generate whole new maps.
	*/
	proc/GenerateNewWorld()

		worldSectors = new/list()

		// Alternately, I could have done this:
		// block(locate(1,1,1),locate(world.maxx/5,world.maxy/5,1)
		// but I had some existing code.

		var/area/sector/secA1 = new()
		var/area/sector/secB1 = new()
		var/area/sector/secC1 = new()
		var/area/sector/secD1 = new()
		var/area/sector/secE1 = new()
		var/area/sector/secA2 = new()
		var/area/sector/secB2 = new()
		var/area/sector/secC2 = new()
		var/area/sector/secD2 = new()
		var/area/sector/secE2 = new()
		var/area/sector/secA3 = new()
		var/area/sector/secB3 = new()
		var/area/sector/secC3 = new()
		var/area/sector/secD3 = new()
		var/area/sector/secE3 = new()
		var/area/sector/secA4 = new()
		var/area/sector/secB4 = new()
		var/area/sector/secC4 = new()
		var/area/sector/secD4 = new()
		var/area/sector/secE4 = new()
		var/area/sector/secA5 = new()
		var/area/sector/secB5 = new()
		var/area/sector/secC5 = new()
		var/area/sector/secD5 = new()
		var/area/sector/secE5 = new()

		BeautificationQueue = new/list()

		// At this point, we break the sectors up into a 5x5 grid of area/sectors.
		for (var/turf/thisTurf in world)

			BeautificationQueue.Add(thisTurf)

			if (thisTurf.x >= 0 && thisTurf.x <= (world.maxx/5 * 1))
				if (thisTurf.y >= 0 && thisTurf.y <= (world.maxy/5 * 1))
					secE1.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 4)
					secD1.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 3)
					secC1.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 2)
					secB1.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 4 && thisTurf.y <= world.maxy)
					secA1.contents.Add(thisTurf)

			if (thisTurf.x > (world.maxx/5) * 1 && thisTurf.x <= (world.maxx/5) * 2)
				if (thisTurf.y >= 0 && thisTurf.y <= (world.maxy/5 * 1))
					secE2.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 4)
					secD2.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 3)
					secC2.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 2)
					secB2.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 4 && thisTurf.y <= world.maxy)
					secA2.contents.Add(thisTurf)

			if (thisTurf.x > (world.maxx/5) * 2 && thisTurf.x <= (world.maxx/5) * 3)
				if (thisTurf.y >= 0 && thisTurf.y <= (world.maxy/5 * 1))
					secE3.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 4)
					secD3.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 3)
					secC3.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 2)
					secB3.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 4 && thisTurf.y <= world.maxy)
					secA3.contents.Add(thisTurf)

			if (thisTurf.x > (world.maxx/5) * 3 && thisTurf.x <= (world.maxx/5) * 4)
				if (thisTurf.y >= 0 && thisTurf.y <= (world.maxy/5 * 1))
					secE4.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 4)
					secD4.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 3)
					secC4.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 2)
					secB4.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 4 && thisTurf.y <= world.maxy)
					secA4.contents.Add(thisTurf)

			if (thisTurf.x > (world.maxx/5) * 4 && thisTurf.x <= world.maxx)
				if (thisTurf.y >= 0 && thisTurf.y <= (world.maxy/5 * 1))
					secE5.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 4)
					secD5.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 3)
					secC5.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 1 && thisTurf.y <= (world.maxy/5) * 2)
					secB5.contents.Add(thisTurf)

				if (thisTurf.y > (world.maxy/5) * 4 && thisTurf.y <= world.maxy)
					secA5.contents.Add(thisTurf)

		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"A1",secA1))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"A2",secA2))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"A3",secA3))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"A4",secA4))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"A5",secA5))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"B1",secB1))
		worldSectors.Add(new/Sector(RENEGADE_ALIGNMENT,"B2",secB2))
		worldSectors.Add(new/Sector(RENEGADE_ALIGNMENT,"B3",secB3))
		worldSectors.Add(new/Sector(RENEGADE_ALIGNMENT,"B4",secB4))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"B5",secB5))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"C1",secC1))
		worldSectors.Add(new/Sector(RENEGADE_ALIGNMENT,"C2",secC2))
		worldSectors.Add(new/Sector(CORPORATE_ALIGNMENT,"C3",secC3))
		worldSectors.Add(new/Sector(RENEGADE_ALIGNMENT,"C4",secC4))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"C5",secC5))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"D1",secD1))
		worldSectors.Add(new/Sector(RENEGADE_ALIGNMENT,"D2",secD2))
		worldSectors.Add(new/Sector(RENEGADE_ALIGNMENT,"D3",secD3))
		worldSectors.Add(new/Sector(RENEGADE_ALIGNMENT,"D4",secD4))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"D5",secD5))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"E1",secE1))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"E2",secE2))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"E3",secE3))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"E4",secE4))
		worldSectors.Add(new/Sector(WILD_ALIGNMENT,"E5",secE5))

		/* Diagnostic of sector composition only.
		spawn()
			sleep(20)
			var/turfCount = 0
			for (var/turf/thisTurf in world)
				turfCount++
			DebugMsg("total world turfs: [turfCount]")
			DebugMsg("Content count by sector:")
			DebugMsg("A1: [secA1.contents.len] A2: [secA2.contents.len] A3: [secA3.contents.len] A4: [secA4.contents.len] A5: [secA5.contents.len]")
			DebugMsg("B1: [secB1.contents.len] B2: [secB2.contents.len] B3: [secB3.contents.len] B4: [secB4.contents.len] B5: [secB5.contents.len]")
			DebugMsg("C1: [secC1.contents.len] C2: [secC2.contents.len] C3: [secC3.contents.len] C4: [secC4.contents.len] C5: [secC5.contents.len]")
			DebugMsg("D1: [secD1.contents.len] D2: [secD2.contents.len] D3: [secD3.contents.len] D4: [secD4.contents.len] D5: [secD5.contents.len]")
			DebugMsg("E1: [secE1.contents.len] E2: [secE2.contents.len] E3: [secE3.contents.len] E4: [secE4.contents.len] E5: [secE5.contents.len]")

		*/

		/* Possible code to use if I introduce dynamic maps

		del worldmaps
		worldmaps = new/list()

		var/swapmap/newMap = new/swapmap("_level1_",DYN_MAPSIZE_X,DYN_MAPSIZE_Y,1)
		FillMapUnderground(newMap)

		worldmaps.Add(newMap)


	proc/FillMapUnderground(var/swapmap/thisMap)

		if (!istype(thisMap,/swapmap))
			DebugMsg("world.FillMapUnderground() requires swapmap arg to populate.")
			return()

		for (var/turf/thisTurf in block(locate(thisMap.x1,thisMap.y1,thisMap.z1),locate(thisMap.x2,thisMap.y2,thisMap.z1)))
			new/turf/wall/soil(thisTurf)


	*/
