/*
	obj/ability serve these purposes:
	1. These are the visible screen elements that should appear on player clients skin controls.
	2. These are the objects which track an individual mob/player's progress (or lack thereof) by residing in their contents.
	3. These may also be objects which reside on a global level that all players use to equal affect.
	4. When Activated(), obj/ability will either add an appropriate Action datum to the mob/action or do something immediately
	   depending on what's appropriate.
	   ** OBJ/ABILITY NO LONGER DO ANYTHING ON THEIR OWN, BUT RATHER EXIST AS ICONS OF THINGS THAT THE PLAYER CAN DO **
	5. Players can bind a given obj/ability to a hotkey by right-clicking on them.

*/

/* By returning fairly outlandish values, I don't need to be informed when an ability cost is undefined. */
var/const/UNDEFINED_ABILITY_MASS_COST = 99999
var/const/UNDEFINED_ABILITY_ENERGY_COST = 99999
var/const/UNDEFINED_ABILITY_PRESTIGE_COST = 99999

client

	var/obj/ability/activeAbility
	var/obj/ability/F1Ability
	var/obj/ability/F2Ability
	var/obj/ability/F3Ability
	var/obj/ability/F4Ability
	var/obj/ability/F5Ability
	var/obj/ability/F6Ability
	var/obj/ability/F7Ability
	var/obj/ability/F8Ability
	var/obj/ability/F9Ability
	var/obj/ability/F10Ability
	var/obj/ability/F11Ability
	var/obj/ability/F12Ability

obj/ability

	icon = 'imgAbilities.dmi'
	name = "None"
	desc = "There's no ability assigned here.  While you could attempt to activate it, nothing will happen."

	var/prestigeCost

	var/actionTypeToQueue // Points to the action that this ability queues.

	var/uses // Current number of uses the player has of this skill.

	New(var/location)

		/*
		if (location != null)
			if (!istype(loc,/mob/player))
				DebugMsg("[type].New(): Abilities must be placed either in a null location or in a player..")
				del src
				return()
		*/

		if (uses == null) uses = 0 // Uses are only assigned to abilities with a temporary nature, which I've yet to get around to.

		BuildIcon()
		..()

	Click()
		var/tmp/mob/player/thisPlayer = usr
		thisPlayer.client.activeAbility = src
		thisPlayer.client.UpdateActiveAbility()
		thisPlayer.client.SkinPurchaseScreenSelectedNewAbility()


	/* Purchase()
		Primarily used by citizens to purchase obj/abilities and upgrade existing ones.
		It's triggered by the btnAbilityPurchaseButton verb. */
	proc/Purchase(var/mob/player/whoIsBuying)

		if (GetPrice() == UNDEFINED_ABILITY_PRESTIGE_COST)
			whoIsBuying.GameOutput("We're sorry, but the [name] ability is not for sale.")
			return()

		if (src.type in whoIsBuying.contents)
			// Later, I might want to implement a means to upgrade.
			whoIsBuying.GameOutput ("You already have the [name] ability.")
			return()

		if (whoIsBuying.GetPrestige() > GetPrice())
			whoIsBuying.AdjustPrestige(-GetPrice(),"purchased [name]")
			var/obj/ability/newAbility = new type(whoIsBuying)
			whoIsBuying.GameOutput ("Congratulations on your [newAbility.name] ability purchase!")
			whoIsBuying.client.UpdateAbilityBank()

	/* GetPrice()
		This is rather complicated because I'm going to tolerate upgrades of abilities. */
	proc/GetPrice()
		if (prestigeCost == null)
			return(UNDEFINED_ABILITY_PRESTIGE_COST)
		else
			return(prestigeCost)

	/* GetMassCost()
		Returns the cost in mass to use this ability. */
	proc/GetMassCost()
		if (actionTypeToQueue != null)
			var/Action/thisAction = locate(actionTypeToQueue)
			if (thisAction != null)
				return(thisAction.GetMassCost())
			else
				thisAction = new actionTypeToQueue(null)
				if (thisAction != null)
					return(thisAction.GetMassCost())
				else
					// DebugMsg("[type].GetMassCost() cannot locate [actionTypeToQueue].")
					return(UNDEFINED_ABILITY_MASS_COST)
		else
			// DebugMsg("[type].GetMassCost() Undefined ability mass cost.")
			return(UNDEFINED_ABILITY_MASS_COST)


	/* GetEnergyCost()
		Returns the cost in energy to use this ability. */
	proc/GetEnergyCost()
		if (actionTypeToQueue != null)
			var/Action/thisAction = locate(actionTypeToQueue)
			if (thisAction != null)
				return(thisAction.GetEnergyCost())
			else
				thisAction = new actionTypeToQueue(null)
				if (thisAction != null)
					return(thisAction.GetEnergyCost())
				else
					// DebugMsg("[type].GetEnergyCost() cannot locate [actionTypeToQueue].")
					return(UNDEFINED_ABILITY_MASS_COST)
		else
			// DebugMsg("[type].GetEnergyCost() Undefined ability energy cost.")
			return(UNDEFINED_ABILITY_MASS_COST)

	/* Activate()

		obj/ability/activate simply spawns a new action of actionTypeToQueue in the action queue.

		QueueNewAction is set to ignore this if the mob is dead. */

	proc/Activate(var/mob/player/thisPlayer,var/target)

		if (actionTypeToQueue != null)
			thisPlayer.QueueNewAction(new actionTypeToQueue(thisPlayer))
			thisPlayer.DrawLatestAction()
		else
			thisPlayer << "Selected ability does not exist for activation."

		return()

	/* BindToHotkeyF1-F12()
		Set the client.F1-F12 program to the appropriate value.
		Call the appropriate UpdateAbilityHotkey(hotkeystring) */

	verb/BindToHotkeyF1()
		set name = "Bind to F1"
		set desc = "Binds this ability to the F1 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F1")

			thisPlayer.client.F1Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F1")


	verb/BindToHotkeyF2()
		set name = "Bind to F2"
		set desc = "Binds this ability to the F2 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F2")

			thisPlayer.client.F2Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F2")

	verb/BindToHotkeyF3()
		set name = "Bind to F3"
		set desc = "Binds this ability to the F3 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F3")

			thisPlayer.client.F3Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F3")

	verb/BindToHotkeyF4()
		set name = "Bind to F4"
		set desc = "Binds this ability to the F4 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F4")

			thisPlayer.client.F4Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F4")

	verb/BindToHotkeyF5()
		set name = "Bind to F5"
		set desc = "Binds this ability to the F5 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F5")

			thisPlayer.client.F5Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F5")

	verb/BindToHotkeyF6()
		set name = "Bind to F6"
		set desc = "Binds this ability to the F6 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F6")

			thisPlayer.client.F6Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F6")

	verb/BindToHotkeyF7()
		set name = "Bind to F7"
		set desc = "Binds this ability to the F7 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F7")

			thisPlayer.client.F7Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F7")

	verb/BindToHotkeyF8()
		set name = "Bind to F8"
		set desc = "Binds this ability to the F8 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F8")

			thisPlayer.client.F8Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F8")

	verb/BindToHotkeyF9()
		set name = "Bind to F9"
		set desc = "Binds this ability to the F9 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F9")

			thisPlayer.client.F9Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F9")

	verb/BindToHotkeyF10()
		set name = "Bind to F10"
		set desc = "Binds this ability to the F10 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F10")

			thisPlayer.client.F10Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F10")

	verb/BindToHotkeyF11()
		set name = "Bind to F11"
		set desc = "Binds this ability to the F11 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F11")

			thisPlayer.client.F11Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F11")

	verb/BindToHotkeyF12()
		set name = "Bind to F12"
		set desc = "Binds this ability to the F2 key for ease of making it active."
		set src in usr.contents

		if (istype(usr,/mob/player))
			var/tmp/mob/player/thisPlayer = usr

			RemoveAbilityFromAllHotkeysExcept(thisPlayer,"F12")

			thisPlayer.client.F12Ability = src
			BuildIcon()
			thisPlayer.client.UpdateAbilityHotkey("F2")

	proc/RemoveAbilityFromAllHotkeysExcept(var/mob/player/whichPlayer,var/whichToNotRemove)

		if (whichToNotRemove != "F1")
			if (src == whichPlayer.client.F1Ability)
				whichPlayer.client.F1Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F1")

		if (whichToNotRemove != "F2")
			if (src == whichPlayer.client.F2Ability)
				whichPlayer.client.F2Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F2")

		if (whichToNotRemove != "F3")
			if (src == whichPlayer.client.F3Ability)
				whichPlayer.client.F3Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F3")

		if (whichToNotRemove != "F4")
			if (src == whichPlayer.client.F4Ability)
				whichPlayer.client.F4Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F4")

		if (whichToNotRemove != "F5")
			if (src == whichPlayer.client.F5Ability)
				whichPlayer.client.F5Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F5")

		if (whichToNotRemove != "F6")
			if (src == whichPlayer.client.F6Ability)
				whichPlayer.client.F6Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F6")

		if (whichToNotRemove != "F7")
			if (src == whichPlayer.client.F7Ability)
				whichPlayer.client.F7Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F7")

		if (whichToNotRemove != "F8")
			if (src == whichPlayer.client.F8Ability)
				whichPlayer.client.F8Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F8")

		if (whichToNotRemove != "F9")
			if (src == whichPlayer.client.F9Ability)
				whichPlayer.client.F9Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F9")

		if (whichToNotRemove != "F10")
			if (src == whichPlayer.client.F10Ability)
				whichPlayer.client.F10Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F10")

		if (whichToNotRemove != "F11")
			if (src == whichPlayer.client.F11Ability)
				whichPlayer.client.F11Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F11")

		if (whichToNotRemove != "F12")
			if (src == whichPlayer.client.F12Ability)
				whichPlayer.client.F12Ability = null
				whichPlayer.client.UpdateAbilityHotkey("F12")


	/* HasUnlimitedAccess()
		This will check the organization that the mob belongs to.  If it is one that grants unlimited access of
		this program to this mob, then it returns 1. */
	proc/HasUnlimitedAccess(var/mob/thisMob)
		// HasUnlimitedAccess faculty has yet to be coded...
		if (uses == -1)
			return(1)
		else
			return(0)


	/* BuildIcon()
	    Rebuilds overlays, underlays, and icon based off of the statistics of this object,  */
	proc/BuildIcon()

		while (overlays.len > 0)
			overlays.Remove(overlays[1])

		while (underlays.len > 0)
			underlays.Remove(underlays[1])

		if (icon_state == null)
			icon_state = "null"
			return()

		// If src != null then it must exist on a player which means it can have uses, be bound to hotkeys, ect.
		if (loc != null)

			var/mob/player/myPlayer = loc

			underlays += icon('imgAbilities.dmi',"background")

			// Asign use-based information.  Either number of uses or an infinity icon.

			if (uses > 99 || HasUnlimitedAccess(loc))
				overlays.Add(icon('imgAbilities.dmi',"unlimited"))

			else

				switch (uses % 10)
					if (1)
						overlays.Add(icon('imgAbilities.dmi',"1"))
					if (2)
						overlays.Add(icon('imgAbilities.dmi',"2"))
					if (3)
						overlays.Add(icon('imgAbilities.dmi',"3"))
					if (4)
						overlays.Add(icon('imgAbilities.dmi',"4"))
					if (5)
						overlays.Add(icon('imgAbilities.dmi',"5"))
					if (6)
						overlays.Add(icon('imgAbilities.dmi',"6"))
					if (7)
						overlays.Add(icon('imgAbilities.dmi',"7"))
					if (8)
						overlays.Add(icon('imgAbilities.dmi',"8"))
					if (9)
						overlays.Add(icon('imgAbilities.dmi',"9"))
					else // Note this also works as our "unrecognized argument" setting
						overlays.Add(icon('imgAbilities.dmi',"0"))

				if (uses > 10)

					switch ((uses-(uses % 10))/10)
						if (1)
							overlays.Add(icon('imgAbilities.dmi',"10"))
						if (2)
							overlays.Add(icon('imgAbilities.dmi',"20"))
						if (3)
							overlays.Add(icon('imgAbilities.dmi',"30"))
						if (4)
							overlays.Add(icon('imgAbilities.dmi',"40"))
						if (5)
							overlays.Add(icon('imgAbilities.dmi',"50"))
						if (6)
							overlays.Add(icon('imgAbilities.dmi',"60"))
						if (7)
							overlays.Add(icon('imgAbilities.dmi',"70"))
						if (8)
							overlays.Add(icon('imgAbilities.dmi',"80"))
						else // Note this also works as our "unrecognized argument" setting
							overlays.Add(icon('imgAbilities.dmi',"90"))

			// Assign hotkey icon if appropriate

			if (myPlayer.client.F1Ability == src)
				overlays.Add(GLOBAL_ICON_F1_OVERLAY)
			else if (myPlayer.client.F2Ability == src)
				overlays.Add(GLOBAL_ICON_F2_OVERLAY)
			else if (myPlayer.client.F3Ability == src)
				overlays.Add(GLOBAL_ICON_F3_OVERLAY)
			else if (myPlayer.client.F4Ability == src)
				overlays.Add(GLOBAL_ICON_F4_OVERLAY)
			else if (myPlayer.client.F5Ability == src)
				overlays.Add(GLOBAL_ICON_F5_OVERLAY)
			else if (myPlayer.client.F6Ability == src)
				overlays.Add(GLOBAL_ICON_F6_OVERLAY)
			else if (myPlayer.client.F7Ability == src)
				overlays.Add(GLOBAL_ICON_F7_OVERLAY)
			else if (myPlayer.client.F8Ability == src)
				overlays.Add(GLOBAL_ICON_F8_OVERLAY)
			else if (myPlayer.client.F9Ability == src)
				overlays.Add(GLOBAL_ICON_F9_OVERLAY)
			else if (myPlayer.client.F10Ability == src)
				overlays.Add(GLOBAL_ICON_F10_OVERLAY)
			else if (myPlayer.client.F11Ability == src)
				overlays.Add(GLOBAL_ICON_F11_OVERLAY)
			else if (myPlayer.client.F12Ability == src)
				overlays.Add(GLOBAL_ICON_F12_OVERLAY)