/*
	Devices are objects which run off of power to perform some function or another.

	Devices do nothing but soak up stray energy bolts unless they are in a Sector_datum's contents.  At that point,
	the magic starts: the devices' heartbeats are ran once per turn.

*/

// These are set on the powered variable to both determine if the heartbeat is running and if it should run.

var/const/DEFAULT_DEVICE_ENERGY = 0
var/const/DEFAULT_DEVICE_MAX_ENERGY = 100
var/const/WIRELESS_ENERGY_TRANSMITTER_MAX_RANGE = 20

obj/device
	icon = 'imgDevices.dmi'

	var/energy
	var/maxEnergy

	Click(var/location,var/control,var/params)
		if (usr.client != null)
			if (src.type != /obj/device/spawnchamber)
				if (!(src in view(usr)))
					usr.GameOutput("You can't see that.")
					return()

			usr.client.statobj = src
		..()

	DblClick(var/location,var/control,var/params)
		var/tmp/turf/thisTurf = src.loc

		thisTurf.DblClick(location,control,params)

		..()

	New(var/location, var/initialEnergy)
		if (initialEnergy != null)
			energy = initialEnergy
		else
			energy = DEFAULT_DEVICE_ENERGY

		if (maxEnergy == null)
			maxEnergy = DEFAULT_DEVICE_MAX_ENERGY

		..()

	/*
		Heartbeat()
		The devices heartbeat is seperate from its activation process now in that Activate() functions to kick it off and
		Heartbeat() will be spawned by Activate() if it has not been activated yet.  Again, all devices have their own
		type of this.
	*/
	proc/Heartbeat()
		if (IsActivated())
			UpdateAppearance()

	/* AdjustEnergy()
		Does a clean modification of the energy amount, not permitting it to exceed GetMaxEnergy() or be less than 0 */
	proc/AdjustEnergy(var/amountToAdjustBy)
		// DebugMsg("AdjustEnergy:[amountToAdjustBy]")
		energy = energy + amountToAdjustBy
		if (energy < 0)
			energy = 0
		if (energy > GetMaxEnergy())
			energy = GetMaxEnergy()
		// energy = min(energy,GetMaxEnergy())
		// energy = max(energy,0)
		// Do not attempt to activate anything.

	/* Damage()
		Like the other damage() procs at the time of this writing, Damage() takes an Attack datum.
		Current implementation I'm thinking of having is to have devices automatically pull from their turf and they are
		only destroyed when the turf is destroyed.

		Note that shield generators will do their own calculations before passing the Attack proc back to here.*/
	proc/Damage (var/Attack/thisAttack)
		if (src.loc == null || !istype(loc,/turf))
			return()
		else
			var/turf/myTurf = loc
			var/turftype = loc.type
			myTurf.Damage(thisAttack)
				// I'm basically using a change of the type of turf to indicate that it was destroyed.
			if (loc.type != turftype)
				Die()

	proc/IsActivated()
		if (GetEnergy() > 0)
			return(1)
		else
			return(0)

	/* Die()
		Leaving behind some kind of shrapnal would be nice.  But, failing this, just delete the obj/device.
		Such is the wonders of object oriented programming that I could always override Die to leave the
		appropriate shrapnal. */
	proc/Die()
		del src

	/* GetAlignment()
		Presumes devices are aligned based off of the sector they are placed in.  Polls all areas to find this device.
		Returns null if the device isn't found or the sector has no type.  Either should be considered an error state. */
	proc/GetAlignment()

		if (mySector == null)
			return(UNDEFINED_ALIGNMENT)
		else
			/*
			if (type == /obj/device/spawnchamber)
				DebugMsg("[name]'s sector is [mySector.name]")
			*/
			return(mySector.alignment)

	/* IsFriendly()
		Returns 1 if is located in a friendly AI sector, else returns 0. */
	proc/IsFriendly()
		switch (GetAlignment())
			if (null)
				return(0)
			if (WILD_ALIGNMENT)
				return (0)
			if (RENEGADE_ALIGNMENT)
				return (0)
			if (CORPORATE_ALIGNMENT)
				return(1)

	/* IsHostile()
		Returns the opposite of IsFriendly() */
	proc/IsHostile()
		if (IsFriendly() == 1)
			return(0)
		else
			return(1)

	/* Returns the maximum energy capacity of this device */
	proc/GetMaxEnergy()
		if (maxEnergy == null) return(100)
		else return(maxEnergy)

	proc/GetEnergy()
		if (energy == null) return(0)
		else return(energy)

	proc/Jolt(var/amtToCharge)
		if (amtToCharge == null)
			DebugMsg("[type].Jolt null charge passed.")
			return()

		if (amtToCharge <= 0)
			DebugMsg("[type].Jolt() [amtToCharge] charge passed.")
			return()

		AdjustEnergy(amtToCharge)

