/*
	Autoturrets are probably the most processor-intensive of devices, actively scanning each turn for opponents to fire upon.
	They're effectively actual combatants, which is unusual because usally that only applies to mobs.
*/

obj/device/autoturret
	var/const/MAXIMUM_STORED_TURRET_ENERGY = 100
	var/const/SUPPLIED_ENERGY_GAIN_PER_TURN = 10
	var/const/ENERGY_COST_PER_SHOT = 10
	var/const/DAMAGE_PER_SHOT = 10
	var/const/NATURAL_BATTERY_DRAIN = 2
	var/const/TURRET_DETECTION_RANGE = 5

	var/const/TURRET_DEPLOYING_SOUND = 'Mechanic-Public_D-139.wav'
	var/const/TURRET_FIRING_SOUND = 'Railgunshot.wav'

	var/const/MAXIMUM_FLECHETTE_RANGE = 100

	name = "Autoturret"
	desc = "These turrets are capable of intelligently identifying and firing upon any enemies that they can see."

	icon_state = "activated_deployed"
	density = 0
	invisibility = 100

	var/isDeployed = 0

	icon = 'imgDeviceTurret.dmi'

	New(var/location, var/initialEnergy)

		if (initialEnergy != null) energy = initialEnergy

		maxEnergy = MAXIMUM_STORED_TURRET_ENERGY

		isDeployed = 0
		icon_state = "activated_undeployed"

		UpdateAppearance()

		dir = SOUTH

		..(location, initialEnergy)


	/* AutoTurret probably has the most complex Heartbeat() of the other devices because, while it is active, it is
	   actively scanning for things to shoot. */
	Heartbeat()

		if (IsActivated())

			var/tmp/list/atom/targetList

			if (src.IsHostile())
				targetList = GetCorporateTargets(src)
			else
				targetList = GetRenegadeTargets(src)

			// Found a target.  Stick some holes in it by spawning the appropriate projectile.

			if (targetList == null)
				if (isDeployed == 1)
					Undeploy()

			else

				if (isDeployed == 0)
					Deploy()
				else
					// Get closest range target.
					var/tmp/atom/bestTarget
					for (var/atom/thisTarget in targetList)
						if (istype(thisTarget,/area))
							continue
						if (bestTarget == null || get_dist(thisTarget,src) < get_dist(thisTarget,bestTarget))
							bestTarget = thisTarget

					// dir = get_dir (src,bestTarget)

					if (GetEnergy() > ENERGY_COST_PER_SHOT)

						Noise(src,TURRET_FIRING_SOUND,100)
						dir = get_dir(src,bestTarget)
						flick("firing",src)
						new/obj/projectile/fletchette(loc,bestTarget,DAMAGE_PER_SHOT,MAXIMUM_FLECHETTE_RANGE,src)

						AdjustEnergy(-ENERGY_COST_PER_SHOT)

		..()

	UpdateAppearance()

		if (src.IsHostile())
			if (icon != 'imgDeviceTurretRenegade.dmi')
				icon = 'imgDeviceTurretRenegade.dmi'
		else
			if (icon != 'imgDeviceTurret.dmi')
				icon = 'imgDeviceTurret.dmi'

		if (IsActivated())
			if (isDeployed)
				icon_state = "activated_deployed"
			else
				icon_state = "activated_undeployed"
		else
			if (isDeployed)
				icon_state = "deactivated_deployed"
			else
				icon_state = "deactivated_undeployed"

		/* GetRenegadeTargets(var/source)
			Returns a new list of targets within line of sight which have the isHostile() flag.
			Returns null if there are no targets. */
	proc/GetRenegadeTargets(var/atom/source)

		var/tmp/list/atom/targetList = new/list()

		for (var/mob/thisMob in view(TURRET_DETECTION_RANGE,source))
			if (thisMob.GetAlignment() == RENEGADE_ALIGNMENT)
				targetList.Add(thisMob)

		if (targetList.len == 0)
			return(null)
		else
			return(targetList)

		/* GetCorporateTargets(var/atom/source)
			returns a new list of targets within line of sight which do not have the IsHostile() flag.
			Note that aliens are returned regardless because this code is basically intended to be used by
			turrets under hostile AI control. */
	proc/GetCorporateTargets(var/atom/source)

		var/tmp/list/atom/targetList = new/list()

		for (var/mob/thisMob in view(TURRET_DETECTION_RANGE,source))
			if (thisMob.GetAlignment() == CORPORATE_ALIGNMENT)
				targetList.Add(thisMob)

		if (targetList.len == 0)
			return(null)
		else
			return(targetList)

		/* By Default, turrets are invisible and have no density, but the Deploy command changes that. */
	proc/Deploy()
		invisibility = 0
		density = 1
		// Knock anyone standing on the turret away.
		flick("deploying",src)
		icon_state = "deployed"
		isDeployed = 1
		Noise(src,TURRET_DEPLOYING_SOUND,100)

	proc/Undeploy()
		density = 1
		flick("undeploying",src)

		spawn()
			sleep(10)
			invisibility = 100

		isDeployed = 0

obj/effects/turretfiring

	New(var/location,var/dir)

		dir = dir

		icon = 'imgDeviceTurret.dmi'
		icon_state = "firing"

		spawn()
			sleep(10)
			del src
			..()
