/*
	Sensors are basically lights, they cast luminosity when powered.

	I refer to them as sensors primarily because they are used by autoturrets as well as players.

	As the simplist of devices, they can be used as a baseline to see how the powered variable works in conjunction with
	Activate() and Heartbeat()
*/


obj/device/sensor

	var/const/LUMINOSITY_CAST_BY_SENSOR = 4
	var/const/MAXIMUM_SENSOR_ENERGY = 10
	var/const/ENERGY_DRAINED_PER_10_MS = 1
		// For optimal efficiency, set at MAXIMUM_SENSOR_ENERGY * (ENERGY_DRAINED_PER_10_MS*10)
	var/const/FREQUENCY_OF_SENSOR_POWER_CHECK_IN_MS = 100

	name = "Sensor"
	desc = "Sensors operate like lights, illuminating the area, for a relatively low energy cost."
	icon_state = "sensor_inactive"

	luminosity = 0

	density = 0

	New(var/location, var/initialEnergy)
		maxEnergy = MAXIMUM_SENSOR_ENERGY
		..(location, initialEnergy)

	UpdateAppearance()
		if (IsActivated())
			if (icon_state == "sensor_inactive")
				overlays.Add(GLOBAL_SENSOR_LIT_OVERLAY)
				icon_state = "sensor_active"
				flick("sensor_activating",src)
				luminosity = LUMINOSITY_CAST_BY_SENSOR
		else
			if (icon_state == "sensor_active")
				overlays.Remove(GLOBAL_SENSOR_LIT_OVERLAY)
				icon_state = "sensor_inactive"
				flick("sensor_deactivating",src)
				luminosity = 0
