/*
	Shield generators maintain an energy shield that blocks projectiles and prevents hostiles from passing through it.
*/
/obj/device/shieldgen

	var/const/MAXIMUM_SHIELD_GENERATOR_ENERGY = 250 // This is also MAXIMUM_SHIELD_LEVEL_5
	var/const/MAXIMUM_SHIELD_LEVEL_1 = 50
	var/const/MAXIMUM_SHIELD_LEVEL_2 = 100
	var/const/MAXIMUM_SHIELD_LEVEL_3 = 150
	var/const/MAXIMUM_SHIELD_LEVEL_4 = 200
	var/const/SUPPLIED_SHIELD_ENERGY_GENERATED_PER_TURN = 10

	name = "Shield generator"
	desc = "Shield generators project an energy field which is capable of selectively counteracting hostile forces."
	icon_state = "shieldgendown"

	density = 0

	New(var/location, var/initialEnergy)

		maxEnergy = MAXIMUM_SHIELD_GENERATOR_ENERGY

		..(location, initialEnergy)

	/* Shields handle damage uniquely in that their energy is drained before they fall and start to take damage.
	   After they run out of energy, the (remaining) attack is passed to the default obj/device Damage() proc. */
	Damage (var/Attack/thisAttack)

		if (thisAttack.power >= GetEnergy())
			thisAttack.power = thisAttack.power - GetEnergy()
			energy = 0
			UpdateAppearance()
			if (thisAttack.power > 0)
				..(thisAttack)
		else
			AdjustEnergy(-thisAttack.power)
			UpdateAppearance()


	UpdateAppearance()

		switch (energy)
			if (0)
				icon_state = "shieldgendown"
				luminosity = 0
			if (1 to MAXIMUM_SHIELD_LEVEL_1)
				icon_state = "shieldgen1"
				luminosity = 1
			if (MAXIMUM_SHIELD_LEVEL_1 to MAXIMUM_SHIELD_LEVEL_2)
				icon_state = "shieldgen2"
				luminosity = 1
			if (MAXIMUM_SHIELD_LEVEL_2 to MAXIMUM_SHIELD_LEVEL_3)
				icon_state = "shieldgen3"
				luminosity = 1
			if (MAXIMUM_SHIELD_LEVEL_3 to MAXIMUM_SHIELD_LEVEL_4)
				icon_state = "shieldgen4"
				luminosity = 2
			else
				icon_state = "shieldgen5"
				luminosity = 2

		..()

	proc/IsAccessPermittedFor(var/atom/movable/thisAtom)
		if (IsActivated())

			if (istype(thisAtom,/mob))
				var/mob/thisMob = thisAtom
				if (thisMob.GetAlignment() == src.GetAlignment())
					return(1)
				else
					return(0)


			if (istype(thisAtom,/obj/projectile))
				var/obj/projectile/thisProjectile = thisAtom
				if (thisProjectile.responsibleAtom != null)
					if (istype(thisProjectile.responsibleAtom,/mob))
						var/mob/thisMob = thisProjectile.responsibleAtom
						if (thisMob.GetAlignment() == src.GetAlignment())
							return(1)
						else
							return(0)
					if (istype(thisProjectile.responsibleAtom,/obj/device))
						var/obj/device/thisDevice = thisProjectile.responsibleAtom
						if (thisDevice.GetAlignment() == src.GetAlignment())
							return(1)
						else
							return(0)

		else // When not activated, the shield bars nothing.
			return(1)

		return(1)

