/*
	Used by certain actions, the capacitor functions assume the mob has a built-in capacitor that needs to be charged in
	order to perform the given action.
*/

var/const/DEFAULT_CAPACITOR_CHARGE_ENERGY = 10
var/const/DEFAULT_CAPACITOR_CHARGE_MASS = 2

mob

	var/capacitorChargeEnergy
	var/capacitorChargeEnergyRequired
	var/capacitorChargeMass
	var/capacitorChargeMassRequired

	var/capacitorChargePerTurnEnergy
	var/capacitorChargePerTurnMass

	/* IsCapacitorCharged() returns 1 if the chargedCapacitor level equals the required amount for the ability to function. */
	proc/IsCapacitorCharged()

		// For the time being, only mob/players need to worry about having a capacitor.
		if (src.type != /mob/player)
			return(1)

		if (GetCapacitorEnergy() >= GetCapacitorEnergyRequired())
			return(1)
		else
			return(0)

	/* Returns the current values of the capacitor. */
	proc/GetCapacitorEnergy()

		if (capacitorChargeEnergy == null)
			return(0)
		else
			return(capacitorChargeEnergy)

	proc/GetCapacitorMass()

		if (capacitorChargeMass == null)
			return(0)
		else
			return(capacitorChargeMass)

	/* Returns the required values of the capacitor (set by Action/Ability/Attempt()) */

	proc/GetCapacitorEnergyRequired()
		if (capacitorChargeEnergyRequired == null)
			return(0)
		else
			return(capacitorChargeEnergyRequired)

	proc/GetCapacitorMassRequired()
		if (capacitorChargeMassRequired == null)
			return(0)
		else
			return(capacitorChargeMassRequired)

	/* How quickly the capacitor will charge. */

	proc/GetCapacitorChargePerTurnEnergy()
		if (capacitorChargePerTurnEnergy == null)
			return(DEFAULT_CAPACITOR_CHARGE_ENERGY)
		else
			return (capacitorChargePerTurnEnergy)

	proc/GetCapacitorChargePerTurnMass()
		if (capacitorChargePerTurnMass == null)
			return(DEFAULT_CAPACITOR_CHARGE_MASS)
		else
			return (capacitorChargePerTurnMass)

	/* SetCapacitorFromAction()

		Sets capacitorChargeEnergyRequired and capacitorChargeMassRequired based off of the values of an ability. */

	proc/SetCapacitorFromAction(var/Action/thisAbility)

		capacitorChargeEnergyRequired = thisAbility.GetEnergyCost()
		capacitorChargeMassRequired = thisAbility.GetMassCost()
		// DebugMsg("SetCapacitorFromAction(): Capacitor set.")

	/* ChargeCapacitor()

		Charges the capacitor towards capacitorChargeEnergyRequired and capacitorChargeMassRequired by the appropriate rate. */
	proc/ChargeCapacitor()

		if (GetCapacitorEnergy() <  GetCapacitorEnergyRequired())
			capacitorChargeEnergy = GetCapacitorEnergy() + GetCapacitorChargePerTurnEnergy()

		if (GetCapacitorMass() < GetCapacitorMassRequired())
			capacitorChargeMass = GetCapacitorMass() + GetCapacitorChargePerTurnMass()

	/* DischargeCapacitor()

		The capacitor's all used up now. */
	proc/DischargeCapacitor()

		capacitorChargeEnergy = 0
		capacitorChargeEnergyRequired = 0
		capacitorChargeMass = 0
		capacitorChargeMassRequired = 0